@@ -27,7 +27,6 @@
 #include <osgUtil/LineSegmentIntersector>
 #include <osgEarth/MapNode>
 #include <osgEarth/TerrainEngineNode>
-#include <osgEarth/ElevationQuery>
 #include <osgEarth/StringUtils>
 #include <osgEarth/Terrain>
 #include <osgEarth/VerticalDatum>
@@ -35,7 +34,7 @@
 #include <osgEarthUtil/Controls>
 #include <osgEarthUtil/LatLongFormatter>
 #include <osgEarthUtil/ExampleResources>
-#include <osgEarthAnnotation/PlaceNode>
+#include <osgEarthAnnotation/ModelNode>
 #include <iomanip>
 
 using namespace osgEarth;
@@ -51,7 +50,7 @@ static LabelControl*  s_haeLabel    = 0L;
 static LabelControl*  s_egm96Label  = 0L;
 static LabelControl*  s_mapLabel    = 0L;
 static LabelControl*  s_resLabel    = 0L;
-static PlaceNode*     s_marker      = 0L;
+static ModelNode*     s_marker      = 0L;
 
 
 // An event handler that will print out the elevation at the clicked point
@@ -59,13 +58,11 @@ struct QueryElevationHandler : public osgGA::GUIEventHandler
 {
     QueryElevationHandler()
         : _mouseDown( false ),
-          _terrain  ( s_mapNode->getTerrain() ),
-          _query    ( s_mapNode->getMap() )
+          _terrain  ( s_mapNode->getTerrain() )
     {
         _map = s_mapNode->getMap();
-        _query.setMaxTilesToCache(10);
-        _query.setFallBackOnNoData( false );
         _path.push_back( s_mapNode->getTerrainEngine() );
+        _envelope = _map->getElevationPool()->createEnvelope(_map->getSRS(), 20u);
     }
 
     void update( float x, float y, osgViewer::View* view )
@@ -84,20 +81,20 @@ struct QueryElevationHandler : public osgGA::GUIEventHandler
             mapPoint.fromWorld( _terrain->getSRS(), world );
 
             // do an elevation query:
-            double query_resolution = 0; // max.
-            double out_hamsl        = 0.0;
-            double out_resolution   = 0.0;
+            double query_resolution  = 0.0;  // max.
+            double actual_resolution = 0.0;
+            float elevation          = 0.0f;
 
-            bool ok = _query.getElevation( 
-                mapPoint,
-                out_hamsl,
-                query_resolution, 
-                &out_resolution );
+            std::pair<float, float> result = _envelope->getElevationAndResolution(
+                mapPoint.x(), mapPoint.y());
 
-            if ( ok )
+            elevation = result.first;
+            actual_resolution = result.second;
+
+            if ( elevation != NO_DATA_VALUE )
             {
                 // convert to geodetic to get the HAE:
-                mapPoint.z() = out_hamsl;
+                mapPoint.z() = elevation;
                 GeoPoint mapPointGeodetic( s_mapNode->getMapSRS()->getGeodeticSRS(), mapPoint );
 
                 static LatLongFormatter s_f;
@@ -108,9 +105,15 @@ struct QueryElevationHandler : public osgGA::GUIEventHandler
                     << ", " 
                     << s_f.format(mapPointGeodetic.x(), false) );
 
-                s_mslLabel->setText( Stringify() << out_hamsl );
-                s_haeLabel->setText( Stringify() << mapPointGeodetic.z() );
-                s_resLabel->setText( Stringify() << out_resolution );
+                if (s_mapNode->getMapSRS()->isGeographic())
+                {
+                    double metersPerDegree = s_mapNode->getMapSRS()->getEllipsoid()->getRadiusEquator() / 360.0;
+                    actual_resolution *= metersPerDegree * cos(osg::DegreesToRadians(mapPoint.y()));
+                }
+
+                s_mslLabel->setText( Stringify() << elevation << " m" );
+                s_haeLabel->setText( Stringify() << mapPointGeodetic.z() << " m" );
+                s_resLabel->setText( Stringify() << actual_resolution << " m" );
 
                 double egm96z = mapPoint.z();
 
@@ -121,18 +124,23 @@ struct QueryElevationHandler : public osgGA::GUIEventHandler
                     mapPointGeodetic.x(),
                     egm96z);
                 
-                s_egm96Label->setText(Stringify() << egm96z);
+                s_egm96Label->setText(Stringify() << egm96z << " m");
 
                 yes = true;
             }
 
-            // finally, get a normal ISECT HAE point.
+            // now get a normal ISECT HAE point.
             GeoPoint isectPoint;
             isectPoint.fromWorld( _terrain->getSRS()->getGeodeticSRS(), world );
-            s_mapLabel->setText( Stringify() << isectPoint.alt() );
+            s_mapLabel->setText( Stringify() << isectPoint.alt() << " m");
 
             // and move the marker.
             s_marker->setPosition(mapPoint);
+
+            // normal test.
+            osg::Quat q;
+            q.makeRotate(osg::Vec3(0,0,1), hits.begin()->getLocalIntersectNormal());
+            s_marker->setLocalRotation(q);
         }
 
         if (!yes)
@@ -161,8 +169,24 @@ struct QueryElevationHandler : public osgGA::GUIEventHandler
     const Map*       _map;
     const Terrain*   _terrain;
     bool             _mouseDown;
-    ElevationQuery   _query;
     osg::NodePath    _path;
+    osg::ref_ptr<ElevationEnvelope> _envelope;
+};
+
+
+struct ClickToRemoveElevation : public ControlEventHandler
+{
+    void onClick(Control*)
+    {
+        Map* map = s_mapNode->getMap();
+        ElevationLayerVector layers;
+        map->getLayers(layers);
+        map->beginUpdate();
+        for (ElevationLayerVector::iterator i = layers.begin(); i != layers.end(); ++i) {
+            map->removeLayer(i->get());
+        }
+        map->endUpdate();
+    }
 };
 
 
@@ -204,6 +228,7 @@ int main(int argc, char** argv)
     grid->setControl(0,r++,new LabelControl("Scene graph intersection:"));
     grid->setControl(0,r++,new LabelControl("EGM96 elevation:"));
     grid->setControl(0,r++,new LabelControl("Query resolution:"));
+    grid->setControl(0, r++, new ButtonControl("Click to remove all elevation data", new ClickToRemoveElevation()));
 
     r = 1;
     s_posLabel = grid->setControl(1,r++,new LabelControl(""));
@@ -214,11 +239,15 @@ int main(int argc, char** argv)
     s_egm96Label = grid->setControl(1,r++,new LabelControl(""));
     s_resLabel = grid->setControl(1,r++,new LabelControl(""));
 
-    s_marker = new PlaceNode();
-    s_marker->setMapNode( s_mapNode );
-    s_marker->setIconImage(osgDB::readImageFile("../data/placemark32.png"));
+    
+    Style markerStyle;
+    markerStyle.getOrCreate<ModelSymbol>()->url()->setLiteral("../data/axes.osgt.64.scale");
+    markerStyle.getOrCreate<ModelSymbol>()->autoScale() = true;
+    s_marker = new ModelNode(s_mapNode, markerStyle);
+    //s_marker->setMapNode( s_mapNode );
+    //s_marker->setIconImage(osgDB::readImageFile("../data/placemark32.png"));
     s_marker->setDynamic(true);
-    root->addChild( s_marker );
+    s_mapNode->addChild( s_marker );
 
     const SpatialReference* mapSRS = s_mapNode->getMapSRS();
     s_vdaLabel->setText( mapSRS->getVerticalDatum() ? 
