@@ -1,6 +1,6 @@
 /* -*-c++-*- */
 /* osgEarth - Dynamic map generation toolkit for OpenSceneGraph
-* Copyright 2015 Pelican Mapping
+* Copyright 2016 Pelican Mapping
 * http://osgearth.org
 *
 * osgEarth is free software; you can redistribute it and/or modify
@@ -40,9 +40,11 @@
 #include <osgEarthUtil/Controls>
 #include <osgEarthUtil/ExampleResources>
 #include <osgEarthUtil/LogarithmicDepthBuffer>
+#include <osgEarthUtil/ViewFitter>
 #include <osgEarthAnnotation/AnnotationUtils>
 #include <osgEarthAnnotation/LabelNode>
 #include <osgEarthSymbology/Style>
+#include <osgEarth/ScreenSpaceLayout>
 
 using namespace osgEarth::Util;
 using namespace osgEarth::Util::Controls;
@@ -82,7 +84,7 @@ namespace
             "double-click :",      "zoom to point",
             "scroll wheel :",      "zoom in/out",
             "arrows :",            "pan",
-            "1-6 :",               "fly to preset viewpoints",
+            //"1-6 :",               "fly to preset viewpoints",
             "shift-right-mouse :", "locked panning",
             "u :",                 "toggle azimuth lock",
             "o :",                 "toggle perspective/ortho",
@@ -91,9 +93,10 @@ namespace
             "t :",                 "cycle tethermode",
             "b :",                 "break tether",
             "a :",                 "toggle viewpoint arcing",
-            "z :",                 "toggle throwing",
+            "q :",                 "toggle throwing",
             "k :",                 "toggle collision",
-            "L :",                 "toggle log depth buffer"
+            "L :",                 "toggle log depth buffer",
+            ") :",                 "toggle sceen space layout"
         };
 
         Grid* g = new Grid();
@@ -188,6 +191,43 @@ namespace
         osgEarth::Util::LogarithmicDepthBuffer _ldb;
     };
 
+    /**
+     * Toggles screen space layout on the sismulated objects
+     */
+    struct ToggleSSL : public osgGA::GUIEventHandler
+    {
+        ToggleSSL(osg::Group* g, char key) : _group(g), _key(key), _installed(false) { }
+
+        bool handle(const osgGA::GUIEventAdapter& ea, osgGA::GUIActionAdapter& aa)
+        {
+            if (ea.getEventType() == ea.KEYDOWN && ea.getKey() == _key)
+            {
+                if ( !_installed )
+                {
+                    ScreenSpaceLayout::activate(_group->getOrCreateStateSet());
+                }
+                else
+                {
+                    ScreenSpaceLayout::deactivate(_group->getOrCreateStateSet());
+                }
+
+                _installed = !_installed;
+                return true;
+            }
+            return false;
+        }
+
+        void getUsage(osg::ApplicationUsage& usage) const
+        {
+            using namespace std;
+            usage.addKeyboardMouseBinding(string(1, _key), string("Toggle SSL"));
+        }
+
+        char _key;
+        osg::Group* _group;
+        bool _installed;
+    };
+
     /**
      * Handler to toggle "azimuth locking", which locks the camera's relative Azimuth
      * while panning. For example, it can maintain "north-up" as you pan around. The
@@ -481,6 +521,48 @@ namespace
         osg::ref_ptr<EarthManipulator> _manip;
         double _vfov, _ar, _zn, _zf;
     };
+    
+    struct FitViewToPoints : public osgGA::GUIEventHandler
+    {
+        std::vector<GeoPoint> _points;
+        const SpatialReference* _mapSRS;
+
+        FitViewToPoints(char key, EarthManipulator* manip, const SpatialReference* mapSRS)
+            : _key(key), _manip(manip), _mapSRS(mapSRS)
+        {
+            // Set up a list of control points
+            const SpatialReference* srs = SpatialReference::get("wgs84");
+            _points.push_back(GeoPoint(srs, -120, 30, 0));
+            _points.push_back(GeoPoint(srs, -100, 45, 0));
+        }
+
+        bool handle(const osgGA::GUIEventAdapter& ea, osgGA::GUIActionAdapter& aa)
+        {
+            if (ea.getEventType() == ea.KEYDOWN && ea.getKey() == _key)
+            {
+                ViewFitter fitter(_mapSRS, aa.asView()->getCamera());
+                fitter.setBuffer( 100000.0 );
+                Viewpoint vp;
+                if (fitter.createViewpoint(_points, vp))
+                {
+                    _manip->setViewpoint(vp);
+                    aa.requestRedraw();
+                }
+                return true;
+            }
+            return false;
+        }
+
+        void getUsage(osg::ApplicationUsage& usage) const
+        {
+            using namespace std;
+            usage.addKeyboardMouseBinding(string(1, _key), string("FitViewToPoints"));
+        }
+
+        char _key;
+        osg::ref_ptr<EarthManipulator> _manip;
+    };
+        
 
     /**
      * A simple simulator that moves an object around the Earth. We use this to
@@ -493,29 +575,25 @@ namespace
         {
             if ( !model )
             { 
-                _model = AnnotationUtils::createHemisphere(250.0, osg::Vec4(1,.7,.4,1)); //, 90.0f); //AnnotationUtils::createSphere( 250.0, osg::Vec4(1,.7,.4,1) );
+                _model = AnnotationUtils::createHemisphere(250.0, osg::Vec4(1,.7,.4,1));
             }
-            
-            _xform = new GeoTransform();
-            _xform->setTerrain(mapnode->getTerrain());
 
-            _pat = new osg::PositionAttitudeTransform();
-            _pat->addChild( _model );
-
-            _xform->addChild( _pat );
-
-            _cam = new osg::Camera();
-            _cam->setRenderOrder( osg::Camera::NESTED_RENDER, 1 );
-            _cam->addChild( _xform );
+            _geo = new GeoPositionNode(mapnode);
+            _geo->getPositionAttitudeTransform()->addChild(_model);
 
             Style style;
-            style.getOrCreate<TextSymbol>()->size() = 32.0f;
-            style.getOrCreate<TextSymbol>()->declutter() = false;
-            _label = new LabelNode(_mapnode, GeoPoint(), _name, style);
+            TextSymbol* text = style.getOrCreate<TextSymbol>();
+            text->size() = 32.0f;
+            text->declutter() = false;
+            text->pixelOffset()->set(50, 50);
+            
+            _label = new LabelNode(_name, style);
             _label->setDynamic( true );
-            _cam->addChild( _label );
+            _label->setHorizonCulling(false);
 
-            root->addChild( _cam.get() );
+            _geo->getPositionAttitudeTransform()->addChild(_label);
+
+            root->addChild(_geo.get());
         }
 
         bool handle(const osgGA::GUIEventAdapter& ea, osgGA::GUIActionAdapter& aa)
@@ -523,30 +601,29 @@ namespace
             if ( ea.getEventType() == ea.FRAME )
             {
                 double t0 = osg::Timer::instance()->time_s();
-                double t = fmod( t0, 600.0 ) / 600.0;
+                double t = fmod( t0, 6000.0 ) / 6000.0;
                 double lat, lon;
                 GeoMath::interpolate( D2R*_lat0, D2R*_lon0, D2R*_lat1, D2R*_lon1, t, lat, lon );
                 GeoPoint p( SpatialReference::create("wgs84"), R2D*lon, R2D*lat, 2500.0 );
                 double bearing = GeoMath::bearing(D2R*_lat0, D2R*_lon0, lat, lon);
 
                 float a = sin(t0*0.2);
-                bearing += a * 0.5 * osg::PI;
-                float pitch = 0.0; //a * 0.1 * osg::PI;
+                //bearing += a * 0.5 * osg::PI;
+                float pitch = 0.0;
 
-                _xform->setPosition( p );
+                _geo->setPosition(p);
 
-                _pat->setAttitude(
-                    osg::Quat(pitch, osg::Vec3d(1,0,0)) *
-                    osg::Quat(bearing, osg::Vec3d(0,0,-1)));
-
-                _label->setPosition( p );
+                _geo->setLocalRotation(
+                    osg::Quat(pitch, osg::Vec3d(1, 0, 0)) *
+                    osg::Quat(bearing, osg::Vec3d(0, 0, -1)));
             }
             else if ( ea.getEventType() == ea.KEYDOWN )
             {
                 if ( ea.getKey() == _key )
                 {                                
                     Viewpoint vp = _manip->getViewpoint();
-                    vp.setNode( _pat.get() );
+                    //vp.setNode( _pat.get() );
+                    vp.setNode(_model);
                     vp.range() = 25000.0;
                     vp.pitch() = -45.0;
                     _manip->setViewpoint(vp, 2.0);
@@ -560,14 +637,13 @@ namespace
         char                               _key;
         MapNode*                           _mapnode;
         EarthManipulator*                  _manip;
-        osg::ref_ptr<osg::Camera>          _cam;
-        osg::ref_ptr<GeoTransform>         _xform;
-        osg::ref_ptr<osg::PositionAttitudeTransform> _pat;
         double                             _lat0, _lon0, _lat1, _lon1;
         LabelNode*                         _label;
         osg::Node*                         _model;
         float                              _heading;
         float                              _pitch;
+
+        osg::ref_ptr<GeoPositionNode>      _geo;
     };
 }
 
@@ -605,20 +681,23 @@ int main(int argc, char** argv)
     osgEarth::MapNode* mapNode = osgEarth::MapNode::findMapNode( earthNode );
 
     // user model?
-    osg::Node* model = 0L;
+    osg::ref_ptr<osg::Node> model;
     std::string modelFile;
     if (arguments.read("--model", modelFile))
-        model = osgDB::readNodeFile(modelFile);
+        model = osgDB::readRefNodeFile(modelFile + ".osgearth_shadergen");
+
+    osg::Group* sims = new osg::Group();
+    root->addChild( sims );
 
     // Simulator for tethering:
-    Simulator* sim1 = new Simulator(root, manip, mapNode, model, "Thing 1", '8');
+    Simulator* sim1 = new Simulator(sims, manip, mapNode, model.get(), "Thing 1", '8');
     sim1->_lat0 = 55.0;
     sim1->_lon0 = 45.0;
     sim1->_lat1 = -55.0;
     sim1->_lon1 = -45.0;
     viewer.addEventHandler(sim1);
 
-    Simulator* sim2 = new Simulator(root, manip, mapNode, model, "Thing 2", '9');
+    Simulator* sim2 = new Simulator(sims, manip, mapNode, model.get(), "Thing 2", '9');
     sim2->_name = "Thing 2";
     sim2->_lat0 = 54.0;
     sim2->_lon0 = 45.0;
@@ -645,20 +724,28 @@ int main(int argc, char** argv)
         osgGA::GUIEventAdapter::LEFT_MOUSE_BUTTON,
         osgGA::GUIEventAdapter::MODKEY_SHIFT );
 
+    manip->getSettings()->bindMouseClick(
+        EarthManipulator::ACTION_GOTO,
+        osgGA::GUIEventAdapter::LEFT_MOUSE_BUTTON,
+        osgGA::GUIEventAdapter::MODKEY_SHIFT);
+
     manip->getSettings()->setArcViewpointTransitions( true );    
 
     manip->setTetherCallback( new TetherCB() );
     
-    viewer.addEventHandler(new FlyToViewpointHandler( manip ));
+    //viewer.addEventHandler(new FlyToViewpointHandler( manip ));
     viewer.addEventHandler(new LockAzimuthHandler('u', manip));
     viewer.addEventHandler(new ToggleArcViewpointTransitionsHandler('a', manip));
-    viewer.addEventHandler(new ToggleThrowingHandler('z', manip));
+    viewer.addEventHandler(new ToggleThrowingHandler('q', manip));
     viewer.addEventHandler(new ToggleCollisionHandler('k', manip));
     viewer.addEventHandler(new ToggleProjMatrix('o', manip));
     viewer.addEventHandler(new BreakTetherHandler('b', manip));
     viewer.addEventHandler(new CycleTetherMode('t', manip));
     viewer.addEventHandler(new SetPositionOffset(manip));
     viewer.addEventHandler(new ToggleLDB('L'));
+    viewer.addEventHandler(new ToggleSSL(sims, ')'));
+
+    viewer.addEventHandler(new FitViewToPoints('j', manip, mapNode->getMapSRS()));
 
     viewer.getCamera()->setSmallFeatureCullingPixelSize(-1.0f);
 
