@@ -1,6 +1,6 @@
 /* -*-c++-*- */
 /* osgEarth - Dynamic map generation toolkit for OpenSceneGraph
-* Copyright 2015 Pelican Mapping
+* Copyright 2016 Pelican Mapping
 * http://osgearth.org
 *
 * osgEarth is free software; you can redistribute it and/or modify
@@ -20,11 +20,14 @@
 * along with this program.  If not, see <http://www.gnu.org/licenses/>
 */
 #include <osgEarth/DrapingTechnique>
+#include <osgEarth/DrapingCullSet>
 #include <osgEarth/Capabilities>
 #include <osgEarth/Registry>
 #include <osgEarth/ShaderFactory>
 #include <osgEarth/VirtualProgram>
 #include <osgEarth/Shaders>
+#include <osgEarth/CullingUtils>
+#include <osgEarth/Lighting>
 
 #include <osg/BlendFunc>
 #include <osg/TexGen>
@@ -40,8 +43,44 @@ using namespace osgEarth;
 
 //---------------------------------------------------------------------------
 
+#undef  LC
+#define LC "[DrapingCamera] "
+
 namespace
 {
+    /**
+     * A camera that will traverse the per-thread DrapingCullSet instead of
+     * its own children.
+     */
+    class DrapingCamera : public osg::Camera
+    {
+    public:
+        DrapingCamera(DrapingManager& dm) : osg::Camera(), _dm(dm), _camera(0L)
+        {
+            setCullingActive( false );
+        }
+
+    public: // osg::Node
+
+        void accept(osg::NodeVisitor& nv, const osg::Camera* camera)
+        {
+            _camera = camera;
+            osg::Camera::accept( nv );
+        }
+
+        void traverse(osg::NodeVisitor& nv)
+        {
+            DrapingCullSet& cullSet = _dm.get(_camera);
+            cullSet.accept( nv );
+        }
+
+    protected:
+        virtual ~DrapingCamera() { }
+        DrapingManager& _dm;
+        const osg::Camera* _camera;
+    };
+
+
     // Additional per-view data stored by the draping technique.
     struct LocalPerViewData : public osg::Referenced
     {
@@ -49,6 +88,8 @@ namespace
     };
 }
 
+//---------------------------------------------------------------------------
+
 namespace
 {
     struct Line2d
@@ -94,12 +135,7 @@ namespace
 
     // Experimental.
     void optimizeProjectionMatrix(OverlayDecorator::TechRTTParams& params, double maxFarNearRatio)
-    {
-        LocalPerViewData& local = *static_cast<LocalPerViewData*>(params._techniqueData.get());
-
-        //TODO: add this to the local
-        //local._rttLimitZ->set( 0.0f );
-
+    {        
         // t0,t1,t2,t3 will form a polygon that tightly fits the
         // main camera's frustum. Texture near the camera will get
         // more resolution then texture far away.
@@ -279,21 +315,14 @@ namespace
 
         // apply the result to the projection matrix.
         params._rttProjMatrix.postMult( M );
-
-        // btw, this new clip matrix distorts the Z coordinate as
-        // y approaches +1. That can cause bleed-through in a geocentric
-        // terrain from the other side of the globe. To prevent that, sample a 
-        // point at the near plane and record that as the Maximum allowable
-        // Z coordinate; a vertex shader in the RTT camera will enforce this.
-        osg::Vec4d sampleFar = osg::Vec4d(0,1,1,1) * M;
-
-        //TODO: add this to the shader.
-        //local._rttLimitZ->set( (float)sampleFar.z() );
     }
 }
 
 //---------------------------------------------------------------------------
 
+#undef  LC
+#define LC "[DrapingTechnique] "
+
 DrapingTechnique::DrapingTechnique() :
 _textureUnit     ( 1 ),
 _textureSize     ( 1024 ),
@@ -314,47 +343,47 @@ _maxFarNearRatio ( 5.0 )
 bool
 DrapingTechnique::hasData(OverlayDecorator::TechRTTParams& params) const
 {
-    return params._group->getNumChildren() > 0;
+    return getBound(params).valid();
 }
 
-
-void
-DrapingTechnique::reestablish(TerrainEngineNode* engine)
+namespace
 {
-    if ( !_textureUnit.isSet() )
+    // Customized texture class will disable texture filtering when rendering under a pick camera.
+    class DrapingTexture : public osg::Texture2D
     {
-        // apply the user-request texture unit, if applicable:
-        if ( _explicitTextureUnit.isSet() )
+    public:
+        virtual void apply(osg::State& state) const
         {
-            if ( !_textureUnit.isSet() || *_textureUnit != *_explicitTextureUnit )
+            const osg::StateSet::DefineList& defines = state.getDefineMap().currentDefines;
+            if (defines.find("OE_IS_PICK_CAMERA") != defines.end())
             {
-                _textureUnit = *_explicitTextureUnit;
-            }
-        }
-
-        // otherwise, automatically allocate a texture unit if necessary:
-        else if ( !_textureUnit.isSet() )
-        {
-            int texUnit;
-            if ( engine->getResources()->reserveTextureImageUnit(texUnit, "DrapingTechnique") )
-            {
-                _textureUnit = texUnit;
-                OE_INFO << LC << "Reserved texture image unit " << *_textureUnit << std::endl;
+                FilterMode minFilter = _min_filter;
+                FilterMode magFilter = _mag_filter;
+                DrapingTexture* ncThis = const_cast<DrapingTexture*>(this);
+                ncThis->_min_filter = NEAREST;
+                ncThis->_mag_filter = NEAREST;
+                ncThis->dirtyTextureParameters();
+                osg::Texture2D::apply(state);
+                ncThis->_min_filter = minFilter;
+                ncThis->_mag_filter = magFilter;
+                ncThis->dirtyTextureParameters();
             }
             else
             {
-                OE_WARN << LC << "Uh oh, no texture image units available." << std::endl;
+                osg::Texture2D::apply(state);
             }
         }
-    }
+    };
 }
 
-
 void
 DrapingTechnique::setUpCamera(OverlayDecorator::TechRTTParams& params)
 {
+    OE_INFO << LC << "Using texture size = " << _textureSize.get() << std::endl;
+
     // create the projected texture:
-    osg::Texture2D* projTexture = new osg::Texture2D();
+    osg::Texture2D* projTexture = new DrapingTexture(); 
+
     projTexture->setTextureSize( *_textureSize, *_textureSize );
     projTexture->setInternalFormat( GL_RGBA );
     projTexture->setSourceFormat( GL_RGBA );
@@ -367,7 +396,7 @@ DrapingTechnique::setUpCamera(OverlayDecorator::TechRTTParams& params)
     projTexture->setBorderColor( osg::Vec4(0,0,0,0) );
 
     // set up the RTT camera:
-    params._rttCamera = new osg::Camera();
+    params._rttCamera = new DrapingCamera(_drapingManager);
     params._rttCamera->setClearColor( osg::Vec4f(0,0,0,0) );
     // this ref frame causes the RTT to inherit its viewpoint from above (in order to properly
     // process PagedLOD's etc. -- it doesn't affect the perspective of the RTT camera though)
@@ -388,7 +417,7 @@ DrapingTechnique::setUpCamera(OverlayDecorator::TechRTTParams& params)
         // a PBUFFER_RTT impl
         if ( Registry::capabilities().supportsDepthPackedStencilBuffer() )
         {
-#ifdef OSG_GLES2_AVAILABLE 
+#if defined(OSG_GLES2_AVAILABLE) || defined(OSG_GLES3_AVAILABLE)
             params._rttCamera->attach( osg::Camera::PACKED_DEPTH_STENCIL_BUFFER, GL_DEPTH24_STENCIL8_EXT );
 #else
             params._rttCamera->attach( osg::Camera::PACKED_DEPTH_STENCIL_BUFFER, GL_DEPTH_STENCIL_EXT );
@@ -400,20 +429,22 @@ DrapingTechnique::setUpCamera(OverlayDecorator::TechRTTParams& params)
         }
 
         params._rttCamera->setClearStencil( 0 );
-        params._rttCamera->setClearMask( GL_COLOR_BUFFER_BIT | GL_STENCIL_BUFFER_BIT ); //GL_DEPTH_BUFFER_BIT |  );
+        params._rttCamera->setClearMask( GL_COLOR_BUFFER_BIT | GL_STENCIL_BUFFER_BIT );
     }
     else
     {
-        params._rttCamera->setClearMask( GL_COLOR_BUFFER_BIT ); //| GL_DEPTH_BUFFER_BIT );
+        params._rttCamera->setClearMask( GL_COLOR_BUFFER_BIT );
     }
 
+
     // set up a StateSet for the RTT camera.
     osg::StateSet* rttStateSet = params._rttCamera->getOrCreateStateSet();
 
     osg::StateAttribute::OverrideValue forceOff =
         osg::StateAttribute::OFF | osg::StateAttribute::PROTECTED | osg::StateAttribute::OVERRIDE;
 
-    rttStateSet->addUniform( Registry::shaderFactory()->createUniformForGLMode(GL_LIGHTING, forceOff) );
+    rttStateSet->setDefine(OE_LIGHTING_DEFINE, forceOff);
+    //rttStateSet->addUniform( Registry::shaderFactory()->createUniformForGLMode(GL_LIGHTING, forceOff) );
     rttStateSet->setMode( GL_LIGHTING, forceOff );
     
     // activate blending within the RTT camera's FBO
@@ -447,7 +478,7 @@ DrapingTechnique::setUpCamera(OverlayDecorator::TechRTTParams& params)
     // overlay geometry is rendered with no depth testing, and in the order it's found in the
     // scene graph... until further notice.
     rttStateSet->setMode(GL_DEPTH_TEST, 0);
-    rttStateSet->setBinName( "TraversalOrderBin" );
+    rttStateSet->setRenderBinDetails(1, "TraversalOrderBin", osg::StateSet::OVERRIDE_PROTECTED_RENDERBIN_DETAILS );
 
     // add to the terrain stateset, i.e. the stateset that the OverlayDecorator will
     // apply to the terrain before cull-traversing it. This will activate the projective
@@ -458,10 +489,29 @@ DrapingTechnique::setUpCamera(OverlayDecorator::TechRTTParams& params)
     LocalPerViewData* local = new LocalPerViewData();
     params._techniqueData = local;
     
+    if ( _maxFarNearRatio > 1.0 )
+    {
+        // Custom clipper that accounts for the projection matrix warping.
+        // Without this, you will get geometry beyond the original ortho far plane.
+        // (i.e. geometry from beyond the horizon will show through the earth)
+        // When the projection matrix has been warped, verts with Z=1.0 are no longer
+        // always on the far plane (only when y=-1) because of the perspective divide (w).
+        // So we need to test the Z directly (without w) and clip. NOTE: this seems to work
+        // fine, although I think it would be proper to clip at the fragment level with 
+        // alpha. We shall see.
+        const char* warpClip =
+            "#version " GLSL_VERSION_STR "\n"
+            GLSL_DEFAULT_PRECISION_FLOAT "\n"
+            "void oe_overlay_warpClip(inout vec4 vclip) { \n"
+            "    if (vclip.z > 1.0) vclip.z = vclip.w+1.0; \n"
+            "} \n";
+        VirtualProgram* rtt_vp = VirtualProgram::getOrCreate(rttStateSet);
+        rtt_vp->setFunction( "oe_overlay_warpClip", warpClip, ShaderComp::LOCATION_VERTEX_CLIP );
+    }
 
     // Assemble the terrain shaders that will apply projective texturing.
     VirtualProgram* terrain_vp = VirtualProgram::getOrCreate(params._terrainStateSet);
-    terrain_vp->setName( "DrapingTechnique terrain shaders");
+    terrain_vp->setName( "Draping terrain shaders");
 
     // sampler for projected texture:
     params._terrainStateSet->getOrCreateUniform(
@@ -482,12 +532,55 @@ void
 DrapingTechnique::preCullTerrain(OverlayDecorator::TechRTTParams& params,
                                  osgUtil::CullVisitor*             cv )
 {
-    if ( !params._rttCamera.valid() && params._group->getNumChildren() > 0 && _textureUnit.isSet() )
+    // allocate a texture image unit the first time through.
+    if ( !_textureUnit.isSet() )
+    {
+        static Threading::Mutex m;
+        m.lock();
+        if ( !_textureUnit.isSet() )
+        {
+            // apply the user-request texture unit, if applicable:
+            if ( _explicitTextureUnit.isSet() )
+            {
+                if ( !_textureUnit.isSet() || *_textureUnit != *_explicitTextureUnit )
+                {
+                    _textureUnit = *_explicitTextureUnit;
+                }
+            }
+
+            // otherwise, automatically allocate a texture unit if necessary:
+            else if ( !_textureUnit.isSet() )
+            {
+                int texUnit;
+                if ( params._terrainResources->reserveTextureImageUnit(texUnit, "Draping") )
+                {
+                    _textureUnit = texUnit;
+                    OE_INFO << LC << "Reserved texture image unit " << *_textureUnit << std::endl;
+                }
+                else
+                {
+                    OE_WARN << LC << "No texture image units available." << std::endl;
+                }
+            }
+        }
+        m.unlock();
+    }
+
+    if ( !params._rttCamera.valid() && _textureUnit.isSet() )
     {
         setUpCamera( params );
+
+        // We do this so we can detect the RTT's camera's parent for 
+        // things like auto-scaling, picking, and so on.
+        params._rttCamera->setView(cv->getCurrentCamera()->getView());
     }
 }
-
+       
+const osg::BoundingSphere&
+DrapingTechnique::getBound(OverlayDecorator::TechRTTParams& params) const
+{
+    return _drapingManager.get(params._mainCamera).getBound();
+}
 
 void
 DrapingTechnique::cullOverlayGroup(OverlayDecorator::TechRTTParams& params,
@@ -495,6 +588,8 @@ DrapingTechnique::cullOverlayGroup(OverlayDecorator::TechRTTParams& params,
 {
     if ( params._rttCamera.valid() )
     {
+        LocalPerViewData& local = *static_cast<LocalPerViewData*>(params._techniqueData.get());
+
         // this xforms from clip [-1..1] to texture [0..1] space
         static osg::Matrix s_scaleBiasMat = 
             osg::Matrix::translate(1.0,1.0,1.0) * 
@@ -511,8 +606,6 @@ DrapingTechnique::cullOverlayGroup(OverlayDecorator::TechRTTParams& params,
 
         osg::Matrix VPT = params._rttViewMatrix * params._rttProjMatrix * s_scaleBiasMat;
 
-        LocalPerViewData& local = *static_cast<LocalPerViewData*>(params._techniqueData.get());
-
         if ( local._texGenUniform.valid() )
         {
             // premultiply the inv view matrix so we don't have precision problems in the shader 
@@ -533,7 +626,7 @@ DrapingTechnique::cullOverlayGroup(OverlayDecorator::TechRTTParams& params,
         }
 
         // traverse the overlay group (via the RTT camera).
-        params._rttCamera->accept( *cv );
+        static_cast<DrapingCamera*>(params._rttCamera.get())->accept( *cv, cv->getCurrentCamera() );
     }
 }
 
@@ -613,7 +706,6 @@ DrapingTechnique::onInstall( TerrainEngineNode* engine )
         unsigned maxSize = Registry::capabilities().getMaxFastTextureSize();
         _textureSize.init( osg::minimum( 2048u, maxSize ) );
     }
-    OE_INFO << LC << "Using texture size = " << *_textureSize << std::endl;
 }
 
 void
