@@ -18,6 +18,8 @@
  */
 #include <osgEarth/GeoTransform>
 #include <osgEarth/Terrain>
+#include <osgEarth/MapNode>
+#include <osgEarth/NodeUtils>
 
 #define LC "[GeoTransform] "
 
@@ -26,39 +28,39 @@
 using namespace osgEarth;
 
 GeoTransform::GeoTransform() :
-_autoRecompute     ( false ),
-_autoRecomputeReady( false )
+_findTerrain(false),
+_terrainCallbackInstalled(false),
+_autoRecomputeHeights(true),
+_dirtyClamp(false)
 {
    //nop
 }
 
-GeoTransform::GeoTransform(const GeoTransform& rhs,
-                           const osg::CopyOp&  op) :
+GeoTransform::GeoTransform(const GeoTransform& rhs, const osg::CopyOp& op) :
 osg::MatrixTransform(rhs, op)
 {
-    _position           = rhs._position;
-    _terrain            = rhs._terrain.get();
-    _autoRecompute      = rhs._autoRecompute;
-    _autoRecomputeReady = false;
+    _position = rhs._position;
+    _terrain = rhs._terrain.get();
+    _autoRecomputeHeights = rhs._autoRecomputeHeights;
+    _terrainCallbackInstalled = false;
+    _findTerrain = false;
+    _dirtyClamp = rhs._dirtyClamp;
 }
 
 void
 GeoTransform::setTerrain(Terrain* terrain)
 {
     _terrain = terrain;
-
-    // Change in the terrain means we need to recompute the position
-    // if one is set.
-    if ( _position.isValid() )
-        setPosition( _position );
+    setPosition(_position);
 }
 
 void
 GeoTransform::setAutoRecomputeHeights(bool value)
 {
-    if (value != _autoRecompute)
+    if (value != _autoRecomputeHeights)
     {
-        _autoRecompute = value;
+        _autoRecomputeHeights = value;
+        setPosition(_position);
     }
 }
 
@@ -74,26 +76,33 @@ GeoTransform::setPosition(const GeoPoint& position)
     if ( !position.isValid() )
         return false;
 
+    bool result = true;
+
     _position = position;
 
     // relative Z or reprojection require a terrain:
     osg::ref_ptr<Terrain> terrain;
     _terrain.lock(terrain);
 
-    // relative Z requires a terrain:
-    if (position.altitudeMode() == ALTMODE_RELATIVE && !terrain.valid())
+    // If we don't have a pointer to a terrain, schedule an attempt
+    // to find one on the next update traversal.
+    if (!terrain.valid() && !_findTerrain)
     {
-        OE_TEST << LC << "setPosition failed condition 1\n";
-        return false;
+        _findTerrain = true;
+        ADJUST_UPDATE_TRAV_COUNT(this, +1);
     }
 
     GeoPoint p;
 
     // transform into terrain SRS if neccesary:
     if (terrain.valid() && !terrain->getSRS()->isEquivalentTo(position.getSRS()))
+    {
         p = position.transform(terrain->getSRS());
+    }
     else
+    {
         p = position;
+    }
 
     // bail if the transformation failed:
     if ( !p.isValid() )
@@ -102,31 +111,30 @@ GeoTransform::setPosition(const GeoPoint& position)
         return false;
     }
 
-    // convert to absolute height:
-    if ( !p.makeAbsolute(_terrain.get()) )
+    // Convert the point to an absolute Z if necessry. If we don't have
+    // a terrain, skip and hope for the best.
+    if (terrain.valid())
     {
-        OE_TEST << LC << "setPosition failed condition 3\n";
-        return false;
+        result = p.makeAbsolute(terrain.get()) && result;
     }
 
-    // assemble the matrix:
+    // Is this is a relative-Z position, we need to install a terrain callback
+    // so we can recompute the altitude when new terrain tiles become available.
+    if (_position.altitudeMode() == ALTMODE_RELATIVE &&
+        _autoRecomputeHeights &&
+        !_terrainCallbackInstalled &&
+        terrain.valid())
+    {
+        // The Adapter template auto-destructs, so we never need to remote it manually.
+        terrain->addTerrainCallback( new TerrainCallbackAdapter<GeoTransform>(this) );
+        _terrainCallbackInstalled = true;
+    }
+
+    // Finally, assemble the matrix from our position point.
     osg::Matrixd local2world;
     p.createLocalToWorld( local2world );
     this->setMatrix( local2world );
 
-    // install auto-recompute?
-    if (_autoRecompute &&
-        _position.altitudeMode() == ALTMODE_RELATIVE &&
-        !_autoRecomputeReady)
-    {
-        // by using the adapter, there's no need to remove
-        // the callback then this object destructs.
-        terrain->addTerrainCallback(
-           new TerrainCallbackAdapter<GeoTransform>(this) );
-
-        _autoRecomputeReady = true;
-    }
-
     return true;
 }
 
@@ -135,19 +143,53 @@ GeoTransform::onTileAdded(const TileKey&          key,
                           osg::Node*              node,
                           TerrainCallbackContext& context)
 {
-   if (!_position.isValid() || _position.altitudeMode() != ALTMODE_RELATIVE)
-   {
-       OE_TEST << LC << "onTileAdded fail condition 1\n";
-       return;
-   }
+    if (!_dirtyClamp)
+    {
+       if (!_position.isValid() || _position.altitudeMode() != ALTMODE_RELATIVE || !_autoRecomputeHeights)
+       {
+           OE_TEST << LC << "onTileAdded fail condition 1\n";
+           return;
+       }
 
-   if (!key.getExtent().contains(_position))
-   {
-       OE_DEBUG << LC << "onTileAdded fail condition 2\n";
-       return;
-   }
+       if (key.valid() && !key.getExtent().contains(_position))
+       {
+           OE_TEST << LC << "onTileAdded fail condition 2\n";
+           return;
+       }
 
-   setPosition(_position);
+       _dirtyClamp = true;
+       ADJUST_UPDATE_TRAV_COUNT(this, +1);
+    }
+
+    //setPosition(_position);
+}
+
+void
+GeoTransform::traverse(osg::NodeVisitor& nv)
+{
+    if (nv.getVisitorType() == nv.UPDATE_VISITOR)
+    {
+        if (_findTerrain)
+        {
+            MapNode* mapNode = osgEarth::findInNodePath<MapNode>(nv);
+            if (mapNode)
+            {
+                _findTerrain = false;
+                ADJUST_UPDATE_TRAV_COUNT(this, -1);
+                setTerrain(mapNode->getTerrain());
+                OE_DEBUG << LC << "Discovered terrain.\n";
+            }
+        }
+
+        if (_dirtyClamp)
+        {
+            setPosition(_position);
+            _dirtyClamp = false;
+            ADJUST_UPDATE_TRAV_COUNT(this, -1);
+        }
+    }
+
+    osg::MatrixTransform::traverse(nv);
 }
 
 void
@@ -156,6 +198,7 @@ GeoTransform::setComputeMatrixCallback(GeoTransform::ComputeMatrixCallback* cb)
     _computeMatrixCallback = cb;
 }
 
+
 bool
 GeoTransform::ComputeMatrixCallback::computeLocalToWorldMatrix(const GeoTransform* xform, osg::Matrix& m, osg::NodeVisitor* nv) const
 {
