@@ -21,12 +21,17 @@
 #include <osgEarth/Registry>
 #include <osgEarth/Capabilities>
 #include <osgEarth/ShaderFactory>
+#include <osgEarth/Lighting>
 #include <osg/Depth>
 
-#define LC "[ModelLayer] "
+#define LC "[ModelLayer] Layer \"" << getName() << "\" "
 
 using namespace osgEarth;
 
+namespace osgEarth {
+    REGISTER_OSGEARTH_LAYER(model, ModelLayer);
+}
+
 //------------------------------------------------------------------------
 
 namespace
@@ -52,53 +57,40 @@ namespace
 
 //------------------------------------------------------------------------
 
-ModelLayerOptions::ModelLayerOptions( const ConfigOptions& options ) :
-ConfigOptions( options )
+ModelLayerOptions::ModelLayerOptions(const ConfigOptions& options) :
+VisibleLayerOptions( options )
 {
     setDefaults();
     fromConfig( _conf ); 
 }
 
-ModelLayerOptions::ModelLayerOptions( const std::string& name, const ModelSourceOptions& driverOptions ) :
-ConfigOptions()
+ModelLayerOptions::ModelLayerOptions(const std::string& in_name, const ModelSourceOptions& driverOptions) :
+VisibleLayerOptions()
 {
     setDefaults();
     fromConfig( _conf );
-    _name = name;
     _driver = driverOptions;
+    name() = in_name;
 }
 
 ModelLayerOptions::ModelLayerOptions(const ModelLayerOptions& rhs) :
-ConfigOptions(rhs.getConfig())
+VisibleLayerOptions(rhs)
 {
-    _name = optional<std::string>(rhs._name);
     _driver = optional<ModelSourceOptions>(rhs._driver);
-    _enabled = optional<bool>(rhs._enabled);
-    _visible = optional<bool>(rhs._visible);
-    _opacity = optional<float>(rhs._opacity);
     _lighting = optional<bool>(rhs._lighting);
     _maskOptions = optional<MaskSourceOptions>(rhs._maskOptions);
     _maskMinLevel = optional<unsigned>(rhs._maskMinLevel);
     _terrainPatch = optional<bool>(rhs._terrainPatch);
-    _cachePolicy = optional<CachePolicy>(rhs._cachePolicy);
-    _cacheId = optional<std::string>(rhs._cacheId);
 }
 
 ModelLayerOptions& ModelLayerOptions::operator =(const ModelLayerOptions& rhs)
 {
-    ConfigOptions::operator =(rhs);
-
-    _name = optional<std::string>(rhs._name);
+    VisibleLayerOptions::operator =(rhs);
     _driver = optional<ModelSourceOptions>(rhs._driver);
-    _enabled = optional<bool>(rhs._enabled);
-    _visible = optional<bool>(rhs._visible);
-    _opacity = optional<float>(rhs._opacity);
     _lighting = optional<bool>(rhs._lighting);
     _maskOptions = optional<MaskSourceOptions>(rhs._maskOptions);
     _maskMinLevel = optional<unsigned>(rhs._maskMinLevel);
     _terrainPatch = optional<bool>(rhs._terrainPatch);
-    _cachePolicy = optional<CachePolicy>(rhs._cachePolicy);
-    _cacheId = optional<std::string>(rhs._cacheId);
 
     return *this;
 }
@@ -106,40 +98,25 @@ ModelLayerOptions& ModelLayerOptions::operator =(const ModelLayerOptions& rhs)
 void
 ModelLayerOptions::setDefaults()
 {
-    _enabled.init     ( true );
-    _visible.init     ( true );
     _lighting.init    ( true );
-    _opacity.init     ( 1.0f );
     _maskMinLevel.init( 0 );
     _terrainPatch.init( false );
-
-    // Expressly set it here since we want no caching by default on a model layer.
-    _cachePolicy = CachePolicy::NO_CACHE;
 }
 
 Config
 ModelLayerOptions::getConfig() const
 {
-    Config conf = ConfigOptions::newConfig();
+    Config conf = VisibleLayerOptions::getConfig();
+    conf.key() = "model";
 
-    conf.updateIfSet( "name",           _name );
-    conf.updateIfSet( "enabled",        _enabled );
-    conf.updateIfSet( "visible",        _visible );
-    conf.updateIfSet( "lighting",       _lighting );
-    conf.updateIfSet( "opacity",        _opacity );
-    conf.updateIfSet( "mask_min_level", _maskMinLevel );
-    conf.updateIfSet( "patch",          _terrainPatch );  
-
-    conf.updateObjIfSet( "cache_policy", _cachePolicy );  
-    conf.updateIfSet("cacheid", _cacheId);
-
-    // Merge the ModelSource options
-    if ( driver().isSet() )
-        conf.merge( driver()->getConfig() );
+    conf.set( "name",           _name );
+    conf.set( "lighting",       _lighting );
+    conf.set( "mask_min_level", _maskMinLevel );
+    conf.set( "patch",          _terrainPatch );  
 
     // Merge the MaskSource options
-    if ( maskOptions().isSet() )
-        conf.add( "mask", maskOptions()->getConfig() );
+    if ( mask().isSet() )
+        conf.set( "mask", mask()->getConfig() );
 
     return conf;
 }
@@ -147,22 +124,15 @@ ModelLayerOptions::getConfig() const
 void
 ModelLayerOptions::fromConfig( const Config& conf )
 {
-    conf.getIfSet( "name",           _name );
-    conf.getIfSet( "enabled",        _enabled );
-    conf.getIfSet( "visible",        _visible );
     conf.getIfSet( "lighting",       _lighting );
-    conf.getIfSet( "opacity",        _opacity );
     conf.getIfSet( "mask_min_level", _maskMinLevel );
     conf.getIfSet( "patch",          _terrainPatch );
 
-    conf.getObjIfSet( "cache_policy", _cachePolicy );  
-    conf.getIfSet("cacheid", _cacheId);
-
     if ( conf.hasValue("driver") )
         driver() = ModelSourceOptions(conf);
 
     if ( conf.hasChild("mask") )
-        maskOptions() = MaskSourceOptions(conf.child("mask"));
+        mask() = MaskSourceOptions(conf.child("mask"));
 }
 
 void
@@ -174,30 +144,45 @@ ModelLayerOptions::mergeConfig( const Config& conf )
 
 //------------------------------------------------------------------------
 
-ModelLayer::ModelLayer( const ModelLayerOptions& options ) :
-_initOptions( options )
+ModelLayer::ModelLayer() :
+VisibleLayer(&_optionsConcrete),
+_options(&_optionsConcrete)
 {
-    copyOptions();
+    init();
 }
 
-ModelLayer::ModelLayer( const std::string& name, const ModelSourceOptions& options ) :
-_initOptions( ModelLayerOptions( name, options ) )
+ModelLayer::ModelLayer(const ModelLayerOptions& options) :
+VisibleLayer(&_optionsConcrete),
+_options(&_optionsConcrete),
+_optionsConcrete(options)
 {
-    copyOptions();
+    init();
 }
 
-ModelLayer::ModelLayer( const ModelLayerOptions& options, ModelSource* source ) :
-_modelSource( source ),
-_initOptions( options )
+ModelLayer::ModelLayer(const std::string& name, const ModelSourceOptions& options) :
+VisibleLayer(&_optionsConcrete),
+_options(&_optionsConcrete),
+_optionsConcrete(ModelLayerOptions(name, options))
 {
-    copyOptions();
+    init();
 }
 
-ModelLayer::ModelLayer(const std::string& name, osg::Node* node):
-_initOptions( ModelLayerOptions(name) ),
+ModelLayer::ModelLayer(const ModelLayerOptions& options, ModelSource* source) :
+VisibleLayer(&_optionsConcrete),
+_options(&_optionsConcrete),
+_optionsConcrete(options),
+_modelSource( source )
+{
+    init();
+}
+
+ModelLayer::ModelLayer(const std::string& name, osg::Node* node) :
+VisibleLayer(&_optionsConcrete),
+_options(&_optionsConcrete),
+_optionsConcrete(ModelLayerOptions(name)),
 _modelSource( new NodeModelSource(node) )
 {
-    copyOptions();
+    init();
 }
 
 ModelLayer::~ModelLayer()
@@ -205,26 +190,34 @@ ModelLayer::~ModelLayer()
     //nop
 }
 
-void
-ModelLayer::copyOptions()
+Config
+ModelLayer::getConfig() const
 {
-    _runtimeOptions = _initOptions;
+    Config layerConf = Layer::getConfig(); //getModelLayerOptions().getConfig();
+    layerConf.set("name", getName()); // redundant?
+    layerConf.set("driver", options().driver()->getDriver());
+    layerConf.key() = "model";
+    return layerConf;
+}
 
-    _alphaEffect = new AlphaEffect();
-    _alphaEffect->setAlpha( *_initOptions.opacity() );
+void
+ModelLayer::init()
+{
+    VisibleLayer::init();
+    _sgCallbacks = new SceneGraphCallbacks();
 }
 
 const Status&
 ModelLayer::open()
 {
-    if ( !_modelSource.valid() && _initOptions.driver().isSet() )
+    if ( !_modelSource.valid() && options().driver().isSet() )
     {
-        std::string driverName = _initOptions.driver()->getDriver();
+        std::string driverName = options().driver()->getDriver();
 
-        OE_INFO << LC << "Opening model layer \"" << getName() << "\", driver=\"" << driverName << "\"" << std::endl;
+        OE_INFO << LC << "Opening; driver=\"" << driverName << "\"" << std::endl;
         
         // Try to create the model source:
-        _modelSource = ModelSourceFactory::create( *_initOptions.driver() );
+        _modelSource = ModelSourceFactory::create( options().driver().get() );
         if ( _modelSource.valid() )
         {
             _modelSource->setName( this->getName() );
@@ -233,63 +226,64 @@ ModelLayer::open()
             if (modelStatus.isOK())
             {
                 // the mask, if there is one:
-                if ( !_maskSource.valid() && _initOptions.maskOptions().isSet() )
+                if ( !_maskSource.valid() && options().mask().isSet() )
                 {
                     OE_INFO << LC << "...initializing mask, driver=" << driverName << std::endl;
 
-                    _maskSource = MaskSourceFactory::create( *_initOptions.maskOptions() );
+                    _maskSource = MaskSourceFactory::create( options().mask().get() );
                     if ( _maskSource.valid() )
                     {
                         const Status& maskStatus = _maskSource->open(_readOptions.get());
                         if (maskStatus.isError())
                         {
-                            _status = maskStatus;
+                            setStatus(maskStatus);
                         }
                     }
                     else
                     {
-                        _status = Status::Error(Status::ServiceUnavailable, Stringify() << "Cannot find mask driver \"" << _initOptions.maskOptions()->getDriver() << "\"");
+                        setStatus(Status::Error(Status::ServiceUnavailable, Stringify() << "Cannot find mask driver \"" << options().mask()->getDriver() << "\""));
                     }
                 }
             }
             else
             {
                 // propagate the model source's error status
-                _status = modelStatus;
+                setStatus(modelStatus);
             }
         }
         else
         {
-            _status = Status::Error(Status::ServiceUnavailable, Stringify() << "Failed to create driver \"" << driverName << "\"");
+            setStatus(Status::Error(Status::ServiceUnavailable, Stringify() << "Failed to create driver \"" << driverName << "\""));
         }
     }
 
-    return _status;
+    return getStatus();
 }
 
+#if 0
 void
 ModelLayer::setReadOptions(const osgDB::Options* readOptions)
 {
-    _readOptions = Registry::cloneOrCreateOptions(readOptions);
+    Layer::setReadOptions(readOptions);
 
     // Create some local cache settings for this layer:
     CacheSettings* oldSettings = CacheSettings::get(readOptions);
     _cacheSettings = oldSettings ? new CacheSettings(*oldSettings) : new CacheSettings();
 
     // bring in the new policy for this layer if there is one:
-    _cacheSettings->integrateCachePolicy(_initOptions.cachePolicy());
+    _cacheSettings->integrateCachePolicy(options().cachePolicy());
 
     // if caching is a go, install a bin.
     if (_cacheSettings->isCacheEnabled())
     {
         std::string binID;
-        if (_initOptions.cacheId().isSet() && !_initOptions.cacheId()->empty())
+        if (options().cacheId().isSet() && !options().cacheId()->empty())
         {
-            binID = _initOptions.cacheId().get();
+            binID = options().cacheId().get();
         }
         else
         {
-            Config conf = _initOptions.driver()->getConfig();
+            Config conf = options().driver()->getConfig();
             binID = hashToString(conf.toJSON(false));
         }
 
@@ -297,13 +291,13 @@ ModelLayer::setReadOptions(const osgDB::Options* readOptions)
         CacheBin* bin = _cacheSettings->getCache()->addBin(binID);
         if (bin)
         {
-            OE_INFO << LC << "Layer \"" << getName() << "\" cache bin is [" << binID << "]\n";
+            OE_INFO << LC << "Cache bin is [" << binID << "]\n";
             _cacheSettings->setCacheBin( bin );
         }
         else
         {
             // failed to create the bin, so fall back on no cache mode.
-            OE_WARN << LC << "Layer " << getName() << " failed to open a cache bin [" << binID << "], disabling caching\n";
+            OE_WARN << LC << "Failed to open a cache bin [" << binID << "], disabling caching\n";
             _cacheSettings->cachePolicy() = CachePolicy::NO_CACHE;
         }
     }
@@ -311,6 +305,24 @@ ModelLayer::setReadOptions(const osgDB::Options* readOptions)
     // Store it for further propagation!
     _cacheSettings->store(_readOptions.get());
 }
+#endif
+
+std::string
+ModelLayer::getCacheID() const
+{
+    // Customized from the base class to use the driver() config instead of full config:
+    std::string binID;
+    if (options().cacheId().isSet() && !options().cacheId()->empty())
+    {
+        binID = options().cacheId().get();
+    }
+    else
+    {
+        Config conf = options().driver()->getConfig();
+        binID = hashToString(conf.toJSON(false));
+    }
+    return binID;
+}
 
 osg::Node*
 ModelLayer::getSceneGraph(const UID& mapUID) const
@@ -341,13 +353,15 @@ ModelLayer::getOrCreateSceneGraph(const Map*        map,
 
     if ( _modelSource.valid() )
     {
+        _modelSource->setSceneGraphCallbacks(_sgCallbacks.get());
+
         node = _modelSource->createNode( map, progress );
 
         if ( node )
         {
-            if ( _runtimeOptions.lightingEnabled().isSet() )
+            if ( options().lightingEnabled().isSet() )
             {
-                setLightingEnabledNoLock( *_runtimeOptions.lightingEnabled() );
+                setLightingEnabledNoLock( options().lightingEnabled().get() );
             }
 
             _modelSource->sync( _modelSourceRev );
@@ -356,13 +370,17 @@ ModelLayer::getOrCreateSceneGraph(const Map*        map,
             // add a parent group for shaders/effects to attach to without overwriting any model programs directly
             osg::Group* group = new osg::Group();
             group->addChild(node);
-            _alphaEffect->attach( group->getOrCreateStateSet() );
+
+            // assign the layer's stateset to the group.
+            osg::StateSet* groupSS = getOrCreateStateSet();
+            group->setStateSet(groupSS);
+
             node = group;
 
             // Toggle visibility if necessary
-            if ( _runtimeOptions.visible().isSet() )
+            if ( options().visible().isSet() )
             {
-                node->setNodeMask( *_runtimeOptions.visible() ? ~0 : 0 );
+                node->setNodeMask( options().visible().get() ? ~0 : 0 );
             }
 
             // Handle disabling depth testing
@@ -370,6 +388,7 @@ ModelLayer::getOrCreateSceneGraph(const Map*        map,
             {
                 osg::StateSet* ss = node->getOrCreateStateSet();
                 ss->setAttributeAndModes( new osg::Depth( osg::Depth::ALWAYS ) );
+              
                 ss->setRenderBinDetails( 99999, "RenderBin" ); //TODO: configure this bin ...
             }
 
@@ -381,54 +400,13 @@ ModelLayer::getOrCreateSceneGraph(const Map*        map,
     return node;
 }
 
-bool
-ModelLayer::getEnabled() const
+osg::Node*
+ModelLayer::getOrCreateNode()
 {
-    return *_runtimeOptions.enabled();
-}
-
-bool
-ModelLayer::getVisible() const
-{
-    return getEnabled() && *_runtimeOptions.visible();
-}
-
-void
-ModelLayer::setVisible(bool value)
-{
-    if ( _runtimeOptions.visible() != value )
-    {
-        _runtimeOptions.visible() = value;
-
-        _mutex.lock();
-        for(Graphs::iterator i = _graphs.begin(); i != _graphs.end(); ++i)
-        {
-            if ( i->second.valid() )
-                i->second->setNodeMask( value ? ~0 : 0 );
-        }
-        _mutex.unlock();
-
-        fireCallback( &ModelLayerCallback::onVisibleChanged );
-    }
-}
-
-float
-ModelLayer::getOpacity() const
-{
-    return *_runtimeOptions.opacity();
-}
-
-void
-ModelLayer::setOpacity(float opacity)
-{
-    if ( _runtimeOptions.opacity() != opacity )
-    {
-        _runtimeOptions.opacity() = opacity;
-
-        _alphaEffect->setAlpha(opacity);
-
-        fireCallback( &ModelLayerCallback::onOpacityChanged );
-    }
+    if (!_graphs.empty())
+        return _graphs.begin()->second.get();
+    else
+        return 0L;
 }
 
 void
@@ -441,7 +419,7 @@ ModelLayer::setLightingEnabled( bool value )
 void
 ModelLayer::setLightingEnabledNoLock(bool value)
 {
-    _runtimeOptions.lightingEnabled() = value;
+    options().lightingEnabled() = value;
 
     for(Graphs::iterator i = _graphs.begin(); i != _graphs.end(); ++i)
     {
@@ -455,8 +433,7 @@ ModelLayer::setLightingEnabledNoLock(bool value)
 
             if ( Registry::capabilities().supportsGLSL() )
             {
-                stateset->addUniform( Registry::shaderFactory()->createUniformForGLMode(
-                    GL_LIGHTING, value ) );
+                stateset->setDefine(OE_LIGHTING_DEFINE, value);
             }
         }
     }
@@ -465,31 +442,16 @@ ModelLayer::setLightingEnabledNoLock(bool value)
 bool
 ModelLayer::isLightingEnabled() const
 {
-    return *_runtimeOptions.lightingEnabled();
+    return options().lightingEnabled().get();
 }
 
 void
-ModelLayer::addCallback( ModelLayerCallback* cb )
+ModelLayer::fireCallback(ModelLayerCallback::MethodPtr method)
 {
-    _callbacks.push_back( cb );
-}
-
-void
-ModelLayer::removeCallback( ModelLayerCallback* cb )
-{
-    ModelLayerCallbackList::iterator i = std::find( _callbacks.begin(), _callbacks.end(), cb );
-    if ( i != _callbacks.end() ) 
-        _callbacks.erase( i );
-}
-
-
-void
-ModelLayer::fireCallback( ModelLayerCallbackMethodPtr method )
-{
-    for( ModelLayerCallbackList::const_iterator i = _callbacks.begin(); i != _callbacks.end(); ++i )
+    for(CallbackVector::const_iterator i = _callbacks.begin(); i != _callbacks.end(); ++i )
     {
-        ModelLayerCallback* cb = i->get();
-        (cb->*method)( this );
+        ModelLayerCallback* cb = dynamic_cast<ModelLayerCallback*>(i->get());
+        if (cb) (cb->*method)( this );
     }
 }
 
