@@ -281,7 +281,6 @@ namespace
 //---------------------------------------------------------------------------
 
 OverlayDecorator::OverlayDecorator() :
-_useShaders          ( true ),
 _dumpRequested       ( false ),
 _rttTraversalMask    ( ~0 ),
 _maxHorizonDistance  ( DBL_MAX ),
@@ -358,7 +357,8 @@ OverlayDecorator::initializePerViewData( PerViewData& pvd, osg::Camera* cam )
         params._group = _overlayGroups[i].get();
         params._terrainStateSet = pvd._sharedTerrainStateSet.get(); // share it.
         params._horizonDistance = &pvd._sharedHorizonDistance;      // share it.
-        params._terrainResources = _engine->getResources();
+        if (_engine.valid())
+            params._terrainResources = _engine->getResources();
         params._mainCamera = cam;
     }
 }
@@ -370,43 +370,38 @@ OverlayDecorator::setOverlayGraphTraversalMask( unsigned mask )
     _rttTraversalMask = mask;
 }
 
-
 void
-OverlayDecorator::onInstall( TerrainEngineNode* engine )
+OverlayDecorator::setTerrainEngine(TerrainEngineNode* engine)
 {
-    _engine = engine;
-
-    // establish the earth's major axis:
-    MapInfo info(engine->getMap());
-    _isGeocentric = info.isGeocentric();
-    _srs = info.getProfile()->getSRS();
-    _ellipsoid = info.getProfile()->getSRS()->getEllipsoid();
-
-    for(Techniques::iterator t = _techniques.begin(); t != _techniques.end(); ++t )
+    if (engine)
     {
-        t->get()->onInstall( engine );
-    }
-}
+        _engine = engine;
 
+        // establish the earth's major axis:
+        MapInfo info(engine->getMap());
+        _isGeocentric = info.isGeocentric();
+        _srs = info.getProfile()->getSRS();
+        _ellipsoid = info.getProfile()->getSRS()->getEllipsoid();
 
-void
-OverlayDecorator::onUninstall( TerrainEngineNode* engine )
-{
-    for(Techniques::iterator t = _techniques.begin(); t != _techniques.end(); ++t )
-    {
-        t->get()->onUninstall( engine );
+        for(Techniques::iterator t = _techniques.begin(); t != _techniques.end(); ++t )
+        {
+            t->get()->onInstall( engine );
+        }
     }
 
-    _engine = 0L;
+    else
+    {
+        for (Techniques::iterator t = _techniques.begin(); t != _techniques.end(); ++t)
+        {
+            t->get()->onUninstall(engine);
+        }
+    }
 }
 
-
 void
 OverlayDecorator::cullTerrainAndCalculateRTTParams(osgUtil::CullVisitor* cv,
                                                    PerViewData&          pvd)
 {
-    static int s_frame = 1;
-
     osg::Vec3d eye = cv->getViewPoint();
 
     double eyeLen;
@@ -423,7 +418,7 @@ OverlayDecorator::cullTerrainAndCalculateRTTParams(osgUtil::CullVisitor* cv,
 
     OE_TEST << LC << "------- OD CULL ------------------------" << std::endl;
 
-    if ( _isGeocentric )
+    if ( _isGeocentric && _engine.valid() )
     {
         eyeLen = eye.length();
 
@@ -596,55 +591,62 @@ OverlayDecorator::cullTerrainAndCalculateRTTParams(osgUtil::CullVisitor* cv,
     // now copy the RTT matrixes over to the techniques.
     for( unsigned t=0; t<pvd._techParams.size(); ++t )
     {
+        OverlayTechnique* tech = _techniques[t].get();
         TechRTTParams& params = pvd._techParams[t];
 
         // skip empty techniques
-        if ( !_techniques[t]->hasData(params) )
+        if ( !tech->hasData(params) )
             continue;
 
         // slice it to fit the overlay geometry. (this says 'visible' but it's just everything..
         // perhaps we can truly make it visible)
         osgShadow::ConvexPolyhedron visiblePH( frustumPH );
 
-#if 0
-        osg::Polytope frustumPT;
-        frustumPH.getPolytope(frustumPT);
-        ComputeVisibleBounds cvb(frustumPT, MVP);
-        params._group->accept(cvb);
-        const osg::BoundingSphere& visibleOverlayBS = cvb._bs;
-        OE_WARN << "VBS radius = " << visibleOverlayBS.radius() << std::endl;
-#else
-        const osg::BoundingSphere& visibleOverlayBS = _techniques[t]->getBound(params);
-#endif
-        if ( visibleOverlayBS.valid() )
+        // if the technique wishes it, compute the bounds of the geometry and
+        // crop the frustum to fit those bounds. This will result in a tighter
+        // fit if the geometry bounds are smaller than the visible region of terrain.
+        if (tech->optimizeToVisibleBound())
         {
-            // form an axis-aligned polytope around the bounding sphere of the
-            // overlay geometry. Use that to cut the camera frustum polytope.
-            // This will minimize the coverage area and also ensure inclusion
-            // of geometry that falls outside the camera frustum but inside
-            // the overlay area.
-            osg::Polytope visibleOverlayPT;
+#if 0
+            osg::Polytope frustumPT;
+            frustumPH.getPolytope(frustumPT);
+            ComputeVisibleBounds cvb(frustumPT, MVP);
+            params._group->accept(cvb);
+            const osg::BoundingSphere& visibleOverlayBS = cvb._bs;
+            OE_WARN << "VBS radius = " << visibleOverlayBS.radius() << std::endl;
+#else
+            const osg::BoundingSphere& visibleOverlayBS = tech->getBound(params);
+#endif
+            if ( visibleOverlayBS.valid() )
+            {
+                // form an axis-aligned polytope around the bounding sphere of the
+                // overlay geometry. Use that to cut the camera frustum polytope.
+                // This will minimize the coverage area and also ensure inclusion
+                // of geometry that falls outside the camera frustum but inside
+                // the overlay area.
+                osg::Polytope visibleOverlayPT;
 
-            osg::Vec3d tangent(0,0,1);
-            if (fabs(worldUp*tangent) > 0.9999)
-                tangent.set(0,1,0);
+                osg::Vec3d tangent(0,0,1);
+                if (fabs(worldUp*tangent) > 0.9999)
+                    tangent.set(0,1,0);
 
-            osg::Vec3d westVec  = worldUp^tangent; westVec.normalize();
-            osg::Vec3d southVec = worldUp^westVec; southVec.normalize();
-            osg::Vec3d eastVec  = -westVec;
-            osg::Vec3d northVec = -southVec;
+                osg::Vec3d westVec  = worldUp^tangent; westVec.normalize();
+                osg::Vec3d southVec = worldUp^westVec; southVec.normalize();
+                osg::Vec3d eastVec  = -westVec;
+                osg::Vec3d northVec = -southVec;
 
-            osg::Vec3d westPt  = visibleOverlayBS.center() + westVec*visibleOverlayBS.radius();
-            osg::Vec3d eastPt  = visibleOverlayBS.center() + eastVec*visibleOverlayBS.radius();
-            osg::Vec3d northPt = visibleOverlayBS.center() + northVec*visibleOverlayBS.radius();
-            osg::Vec3d southPt = visibleOverlayBS.center() + southVec*visibleOverlayBS.radius();
+                osg::Vec3d westPt  = visibleOverlayBS.center() + westVec*visibleOverlayBS.radius();
+                osg::Vec3d eastPt  = visibleOverlayBS.center() + eastVec*visibleOverlayBS.radius();
+                osg::Vec3d northPt = visibleOverlayBS.center() + northVec*visibleOverlayBS.radius();
+                osg::Vec3d southPt = visibleOverlayBS.center() + southVec*visibleOverlayBS.radius();
 
-            visibleOverlayPT.add(osg::Plane(-westVec,  westPt));
-            visibleOverlayPT.add(osg::Plane(-eastVec,  eastPt));
-            visibleOverlayPT.add(osg::Plane(-southVec, southPt));
-            visibleOverlayPT.add(osg::Plane(-northVec, northPt));
+                visibleOverlayPT.add(osg::Plane(-westVec,  westPt));
+                visibleOverlayPT.add(osg::Plane(-eastVec,  eastPt));
+                visibleOverlayPT.add(osg::Plane(-southVec, southPt));
+                visibleOverlayPT.add(osg::Plane(-northVec, northPt));
 
-            visiblePH.cut( visibleOverlayPT );
+                visiblePH.cut( visibleOverlayPT );
+            }
         }
 
         // for dumping, we want the previous fram's projection matrix
@@ -714,7 +716,7 @@ OverlayDecorator::cullTerrainAndCalculateRTTParams(osgUtil::CullVisitor* cv,
             params._rttViewMatrix.set( rttViewMatrix );
             params._rttProjMatrix.set( rttProjMatrix );
             params._eyeWorld = eye;
-            params._visibleFrustumPH = clampedFrustumPH; //frustumPH;
+            params._visibleFrustumPH = clampedFrustumPH;
         }
 
         // service a "dump" of the polyhedrons for dubugging purposes
@@ -727,17 +729,17 @@ OverlayDecorator::cullTerrainAndCalculateRTTParams(osgUtil::CullVisitor* cv,
             {
                 frustumPH.dumpGeometry(0,0,0,fn);
             }
-            osg::Node* camNode = osgDB::readNodeFile(fn);
+            osg::ref_ptr<osg::Node> camNode = osgDB::readRefNodeFile(fn);
             camNode->setName("camera");
 
             // visible overlay BEFORE cutting:
             //uncutVisiblePH.dumpGeometry(0,0,0,fn,osg::Vec4(0,1,1,1),osg::Vec4(0,1,1,.25));
-            //osg::Node* overlay = osgDB::readNodeFile(fn);
+            //osg::ref_ptr<osg::Node> overlay = osgDB::readRefNodeFile(fn);
             //overlay->setName("overlay");
 
             // visible overlay Polyherdron AFTER cuting:
             visiblePH.dumpGeometry(0,0,0,fn,osg::Vec4(1,.5,1,1),osg::Vec4(1,.5,0,.25));
-            osg::Node* intersection = osgDB::readNodeFile(fn);
+            osg::ref_ptr<osg::Node> intersection = osgDB::readRefNodeFile(fn);
             intersection->setName("intersection");
 
             // RTT frustum:
@@ -750,7 +752,7 @@ OverlayDecorator::cullTerrainAndCalculateRTTParams(osgUtil::CullVisitor* cv,
                 rttPH.transform( inverseMVP, MVP );
                 rttPH.dumpGeometry(0,0,0,fn,osg::Vec4(1,1,0,1),osg::Vec4(1,1,0,0.25));
             }
-            osg::Node* rttNode = osgDB::readNodeFile(fn);
+            osg::ref_ptr<osg::Node> rttNode = osgDB::readRefNodeFile(fn);
             rttNode->setName("rtt");
 
             // EyePoint
