@@ -25,6 +25,8 @@
 #include <osgEarth/ShaderFactory>
 #include <osgEarth/StringUtils>
 #include <osgEarth/URI>
+#include <osgEarth/Lighting>
+#include <osgEarth/VirtualProgram>
 
 #include <osg/Drawable>
 #include <osg/Geode>
@@ -66,7 +68,7 @@ using namespace osgEarth;
 
 // compatibility string for GLES:
 
-#ifdef OSG_GLES2_AVAILABLE
+#if defined(OSG_GLES2_AVAILABLE) || defined(OSG_GLES3_AVAILABLE)
 #   define GLSL_PRECISION "precision mediump float;"
 #   define MEDIUMP        "mediump "
 #   define LOWP           "lowp "
@@ -87,67 +89,71 @@ using namespace osgEarth;
 #define TEXENV_COLOR   "oe_sg_texenvcolor"
 #define TEX_MATRIX     "oe_sg_texmat"
 
-#define VERTEX_FUNCTION   "oe_sg_vert"
-#define FRAGMENT_FUNCTION "oe_sg_frag"
+#define VERTEX_MODEL_FUNCTION "oe_sg_vert_model"
+#define VERTEX_VIEW_FUNCTION  "oe_sg_vert_view"
+#define FRAGMENT_FUNCTION     "oe_sg_frag"
 
 // other stuff
 #define INDENT "    "
 
 //------------------------------------------------------------------------
 
-struct OSGEarthShaderGenPseudoLoader : public osgDB::ReaderWriter
+namespace
 {
-    OSGEarthShaderGenPseudoLoader()
+    struct OSGEarthShaderGenPseudoLoader : public osgDB::ReaderWriter
     {
-        this->supportsExtension( SHADERGEN_PL_EXTENSION, "ShaderGen pseudoloader" );
-    }
+        OSGEarthShaderGenPseudoLoader()
+        {
+            this->supportsExtension( SHADERGEN_PL_EXTENSION, "ShaderGen pseudoloader" );
+        }
 
-    const char* className() const
-    {
-        return "OSGEarth ShaderGen pseudoloader";
-    }
+        const char* className() const
+        {
+            return "OSGEarth ShaderGen pseudoloader";
+        }
 
-    bool acceptsExtension(const std::string& extension) const
-    {
-        return osgDB::equalCaseInsensitive( extension, SHADERGEN_PL_EXTENSION );
-    }
+        bool acceptsExtension(const std::string& extension) const
+        {
+            return osgDB::equalCaseInsensitive( extension, SHADERGEN_PL_EXTENSION );
+        }
 
-    ReadResult readObject(const std::string& filename, const osgDB::Options* options) const
-    {
-        return readNode( filename, options );
-    }
+        ReadResult readObject(const std::string& filename, const osgDB::Options* options) const
+        {
+            return readNode( filename, options );
+        }
 
-    ReadResult readNode(const std::string& filename, const osgDB::Options* options) const
-    {
-        if ( !acceptsExtension(osgDB::getFileExtension(filename)) )
-            return ReadResult::FILE_NOT_HANDLED;
+        ReadResult readNode(const std::string& filename, const osgDB::Options* options) const
+        {
+            if ( !acceptsExtension(osgDB::getFileExtension(filename)) )
+                return ReadResult::FILE_NOT_HANDLED;
 
-        std::string stripped = osgDB::getNameLessExtension(filename);
+            std::string stripped = osgDB::getNameLessExtension(filename);
 
-        OE_INFO << LC << "Loading " << stripped << " from PLOD/Proxy and generating shaders." << std::endl;
+            OE_INFO << LC << "Loading " << stripped << " from PLOD/Proxy and generating shaders." << std::endl;
         
-        osgEarth::ReadResult result = URI(stripped).readNode(options);
-        if ( result.succeeded() && result.getNode() != 0L )
-        {
-            osg::ref_ptr<osg::Node> node = result.releaseNode();
+            osgEarth::ReadResult result = URI(stripped).readNode(options);
+            if ( result.succeeded() && result.getNode() != 0L )
+            {
+                osg::ref_ptr<osg::Node> node = result.releaseNode();
 
-            osgEarth::Registry::shaderGenerator().run(
-                node.get(),
-                osgDB::getSimpleFileName(stripped),
-                Registry::stateSetCache() );
+                osgEarth::Registry::shaderGenerator().run(
+                    node.get(),
+                    osgDB::getSimpleFileName(stripped),
+                    Registry::stateSetCache() );
 
-            return ReadResult( node.release() );
+                return ReadResult( node.release() );
+            }
+
+            else
+            {
+                OE_WARN << LC << "Error loading \"" << stripped << "\": " << result.errorDetail() << "\n";
+                return ReadResult::ERROR_IN_READING_FILE;
+            }
         }
+    };
 
-        else
-        {
-            OE_WARN << LC << "Error loading \"" << stripped << "\": " << result.errorDetail() << "\n";
-            return ReadResult::ERROR_IN_READING_FILE;
-        }
-    }
-};
-
-REGISTER_OSGPLUGIN(SHADERGEN_PL_EXTENSION, OSGEarthShaderGenPseudoLoader)
+    REGISTER_OSGPLUGIN(SHADERGEN_PL_EXTENSION, OSGEarthShaderGenPseudoLoader)
+}
 
 //------------------------------------------------------------------------
 
@@ -188,8 +194,12 @@ namespace
         osg::StateAttribute* _sa;
         unsigned             _unit;
     };
+}
+    
+//------------------------------------------------------------------------
 
-
+namespace
+{
     /**
      * The OSG State extended with mode/attribute accessors.
      */
@@ -280,7 +290,13 @@ namespace
         // ref: https://github.com/openscenegraph/osg/commit/22af59482ac4f727eeed5b97476a3a47d7fe8a69
         bool isModeless(osg::StateAttribute* sa) const
         {
-#if OSG_VERSION_LESS_THAN(3,3,1)            
+
+#if !defined(OSG_GL_FIXED_FUNCTION_AVAILABLE)
+            // No modes in non-ffp
+            return true;
+#endif
+
+#if OSG_VERSION_LESS_THAN(3,3,1)
             return
                 dynamic_cast<osg::Texture2DArray*>(sa) ||
                 dynamic_cast<osg::Texture2DMultisample*>(sa) ||
@@ -343,20 +359,6 @@ ShaderGenerator::ShaderGenerator()
     _duplicateSharedSubgraphs = false;
 }
 
-// pre-3.3.0, NodeVisitor didn't have a copy constructor.
-#if OSG_VERSION_LESS_THAN(3,3,0)
-ShaderGenerator::ShaderGenerator(const ShaderGenerator& rhs, const osg::CopyOp& copy) :
-osg::NodeVisitor         (),
-_active                  (rhs._active),
-_duplicateSharedSubgraphs(rhs._duplicateSharedSubgraphs)
-{
-    _visitorType              = rhs._visitorType;
-    _traversalMode            = rhs._traversalMode;
-    _traversalMask            = rhs._traversalMask;
-    _nodeMaskOverride         = rhs._nodeMaskOverride;
-    _state = new StateEx();
-}
-#else
 ShaderGenerator::ShaderGenerator(const ShaderGenerator& rhs, const osg::CopyOp& copy) :
 osg::NodeVisitor         (rhs, copy),
 _active                  (rhs._active),
@@ -364,7 +366,6 @@ _duplicateSharedSubgraphs(rhs._duplicateSharedSubgraphs)
 {
     _state = new StateEx();
 }
-#endif
 
 void
 ShaderGenerator::setIgnoreHint(osg::Object* object, bool ignore)
@@ -424,6 +425,10 @@ ShaderGenerator::run(osg::Node*         graph,
         // perform GL state sharing
         optimizeStateSharing( graph, cache );
 
+        // generate uniforms and uniform callbacks for lighting and material elements.
+        GenerateGL3LightingUniforms generateUniforms;
+        graph->accept(generateUniforms);
+
         osg::StateSet* stateset = cloneOrCreateStateSet(graph);
 
         // install a blank VP at the top as the default.
@@ -484,6 +489,7 @@ ShaderGenerator::apply(osg::Node& node)
 
     if ( stateset.valid() )
     {
+        disableUnsupportedAttributes(stateset.get());
         _state->popStateSet();
     }
 }
@@ -568,26 +574,25 @@ ShaderGenerator::apply( osg::Geode& node )
 
     if ( stateset.valid() )
     {
+        disableUnsupportedAttributes(stateset.get());
         _state->popStateSet();
     }
 }
 
-#if OSG_VERSION_GREATER_OR_EQUAL(3,3,3)
 void
-ShaderGenerator::apply( osg::Drawable& drawable )
+ShaderGenerator::apply( osg::Drawable& node )
 {
     if ( !_active )
         return;
 
-    if ( ignore(&drawable) )
+    if ( ignore(&node) )
         return;
 
     if ( _duplicateSharedSubgraphs )
-        duplicateSharedNode(drawable);
+        duplicateSharedNode(node);
 
-    apply( &drawable );
+    apply( &node );
 }
-#endif
 
 void 
 ShaderGenerator::apply( osg::Drawable* drawable )
@@ -630,6 +635,7 @@ ShaderGenerator::apply( osg::Drawable* drawable )
 
         if ( ss.valid() )
         {
+            disableUnsupportedAttributes(ss.get());
             _state->popStateSet();
         }
     }
@@ -695,6 +701,7 @@ ShaderGenerator::apply(osg::ClipNode& node)
 {
     static const char* s_clip_source =
         "#version " GLSL_VERSION_STR "\n"
+        GLSL_PRECISION "\n"
         "void oe_sg_set_clipvertex(inout vec4 vertexVIEW)\n"
         "{\n"
         "    gl_ClipVertex = vertexVIEW; \n"
@@ -748,7 +755,8 @@ ShaderGenerator::apply(osgSim::LightPointNode& node)
             replacement->removeTextureAttribute(0, sprite.get());
             node.setStateSet(replacement.get() );
         }
-
+        
+        disableUnsupportedAttributes(stateset.get());
         _state->popStateSet();
     }
 }
@@ -783,8 +791,8 @@ ShaderGenerator::processText(const osg::StateSet* ss, osg::ref_ptr<osg::StateSet
 
     std::string vertSrc =
         "#version " GLSL_VERSION_STR "\n" GLSL_PRECISION "\n"
-        "varying " MEDIUMP "vec4 " TEX_COORD_TEXT ";\n"
-        "void " VERTEX_FUNCTION "(inout vec4 vertexVIEW)\n"
+        "out " MEDIUMP "vec4 " TEX_COORD_TEXT ";\n"
+        "void " VERTEX_MODEL_FUNCTION "(inout vec4 unused)\n"
         "{ \n"
         INDENT TEX_COORD_TEXT " = gl_MultiTexCoord0;\n"
         "} \n";
@@ -792,15 +800,14 @@ ShaderGenerator::processText(const osg::StateSet* ss, osg::ref_ptr<osg::StateSet
     std::string fragSrc =
         "#version " GLSL_VERSION_STR "\n" GLSL_PRECISION "\n"
         "uniform sampler2D " SAMPLER_TEXT ";\n"
-        "varying " MEDIUMP "vec4 " TEX_COORD_TEXT ";\n"
+        "in " MEDIUMP "vec4 " TEX_COORD_TEXT ";\n"
         "void " FRAGMENT_FUNCTION "(inout vec4 color)\n"
         "{ \n"
-        INDENT MEDIUMP "vec4 texel = texture2D(" SAMPLER_TEXT ", " TEX_COORD_TEXT ".xy);\n"
-        //INDENT MEDIUMP "vec4 texel = texture2DLod(" SAMPLER_TEXT ", " TEX_COORD_TEXT ".xy, 0.0);\n"
+        INDENT MEDIUMP "vec4 texel = texture(" SAMPLER_TEXT ", " TEX_COORD_TEXT ".xy);\n"
         INDENT "color.a *= texel.a; \n"
         "}\n";
 
-    vp->setFunction( VERTEX_FUNCTION,   vertSrc, ShaderComp::LOCATION_VERTEX_MODEL, 0.5f );
+    vp->setFunction( VERTEX_MODEL_FUNCTION, vertSrc, ShaderComp::LOCATION_VERTEX_MODEL, 0.5f );
     vp->setFunction( FRAGMENT_FUNCTION, fragSrc, ShaderComp::LOCATION_FRAGMENT_COLORING, 0.5f );
     replacement->getOrCreateUniform( SAMPLER_TEXT, osg::Uniform::SAMPLER_2D )->set( 0 );
 
@@ -833,7 +840,7 @@ ShaderGenerator::processGeometry(const osg::StateSet*         original,
         new osg::StateSet();
 
     // likewise, create a VP that we might populate.
-    osg::ref_ptr<VirtualProgram> vp = VirtualProgram::cloneOrCreate(original, newStateSet);
+    osg::ref_ptr<VirtualProgram> vp = VirtualProgram::cloneOrCreate(original, newStateSet.get());
 
     // we'll set this to true if the new stateset goes into effect and
     // needs to be returned.
@@ -853,7 +860,7 @@ ShaderGenerator::processGeometry(const osg::StateSet*         original,
     {
         needNewStateSet = true;
         osg::StateAttribute::GLModeValue value = current->getMode(GL_LIGHTING);
-        newStateSet->addUniform( Registry::shaderFactory()->createUniformForGLMode(GL_LIGHTING, value) );
+        newStateSet->setDefine(OE_LIGHTING_DEFINE, value);
     }
     
     // start generating the shader source.
@@ -904,23 +911,37 @@ ShaderGenerator::processGeometry(const osg::StateSet*         original,
     {
         std::string version = GLSL_VERSION_STR;
 
-        std::string vertHeadSource;
-        vertHeadSource = buf._vertHead.str();
+        std::string modelHeadSource = buf._modelHead.str();
+        std::string modelBodySource = buf._modelBody.str();
 
-        std::string vertBodySource;
-        vertBodySource = buf._vertBody.str();
-
-
-        if ( !vertHeadSource.empty() || !vertBodySource.empty() )
+        if (!modelHeadSource.empty() && !modelBodySource.empty())
         {
-            std::string vertSource = Stringify()
+            std::string modelSource = Stringify()
                 << "#version " << version << "\n" GLSL_PRECISION "\n"
-                << vertHeadSource
-                << "void " VERTEX_FUNCTION "(inout vec4 vertex_view)\n{\n"
-                << vertBodySource
+                << modelHeadSource
+                << "void " VERTEX_MODEL_FUNCTION "(inout vec4 vertex_model)\n{\n"
+                << modelBodySource
                 << "}\n";
 
-            vp->setFunction(VERTEX_FUNCTION, vertSource, ShaderComp::LOCATION_VERTEX_VIEW, 0.5f);
+            vp->setFunction(VERTEX_MODEL_FUNCTION, modelSource, ShaderComp::LOCATION_VERTEX_MODEL, 0.5f);
+        }
+
+        std::string viewHeadSource;
+        viewHeadSource = buf._viewHead.str();
+
+        std::string viewBodySource;
+        viewBodySource = buf._viewBody.str();
+
+        if ( !viewHeadSource.empty() && !viewBodySource.empty() )
+        {
+            std::string viewSource = Stringify()
+                << "#version " << version << "\n" GLSL_PRECISION "\n"
+                << viewHeadSource
+                << "void " VERTEX_VIEW_FUNCTION "(inout vec4 vertex_view)\n{\n"
+                << viewBodySource
+                << "}\n";
+
+            vp->setFunction(VERTEX_VIEW_FUNCTION, viewSource, ShaderComp::LOCATION_VERTEX_VIEW, 0.5f);
         }
 
 
@@ -930,7 +951,7 @@ ShaderGenerator::processGeometry(const osg::StateSet*         original,
         std::string fragBodySource;
         fragBodySource = buf._fragBody.str();
 
-        if ( !fragHeadSource.empty() || !fragBodySource.empty() )
+        if ( !fragHeadSource.empty() && !fragBodySource.empty() )
         {
             std::string fragSource = Stringify()
                 << "#version " << version << "\n" GLSL_PRECISION "\n"
@@ -962,8 +983,9 @@ ShaderGenerator::apply(osg::Texture*     tex,
 {
    bool ok = true;
 
-   buf._vertHead << "varying " MEDIUMP "vec4 " TEX_COORD << unit << ";\n";
-   buf._fragHead << "varying " MEDIUMP "vec4 " TEX_COORD << unit << ";\n";
+   buf._modelHead << "out " MEDIUMP "vec4 " TEX_COORD << unit << ";\n";
+   buf._viewHead << "out " MEDIUMP "vec4 " TEX_COORD << unit << ";\n";
+   buf._fragHead << "in " MEDIUMP "vec4 " TEX_COORD << unit << ";\n";
 
    apply( texgen, unit, buf );
    apply( texmat, unit, buf );
@@ -1080,7 +1102,7 @@ ShaderGenerator::apply(osg::TexGen* texgen, int unit, GenBuffers& buf)
         switch( texgen->getMode() )
         {
         case osg::TexGen::OBJECT_LINEAR:
-            buf._vertBody
+            buf._modelBody
                 << INDENT "{\n"
                 << INDENT TEX_COORD << unit << " = "
                 <<      "gl_Vertex.x*gl_ObjectPlaneS[" <<unit<< "] + "
@@ -1091,7 +1113,7 @@ ShaderGenerator::apply(osg::TexGen* texgen, int unit, GenBuffers& buf)
             break;
 
         case osg::TexGen::EYE_LINEAR:
-            buf._vertBody
+            buf._viewBody
                 << INDENT "{\n"
                 << INDENT TEX_COORD << unit << " = "
                 <<      "vertex_view.x*gl_EyePlaneS[" <<unit<< "] + "
@@ -1102,9 +1124,9 @@ ShaderGenerator::apply(osg::TexGen* texgen, int unit, GenBuffers& buf)
             break;
 
         case osg::TexGen::SPHERE_MAP:
-            buf._vertHead
-                << "varying vec3 vp_Normal;\n";
-            buf._vertBody 
+            buf._viewHead
+                << "vec3 vp_Normal; // stage global\n";
+            buf._viewBody 
                 << INDENT "{\n" // scope it in case there are > 1
                 << INDENT "vec3 view_vec = normalize(vertex_view.xyz/vertex_view.w); \n"
                 << INDENT "vec3 r = reflect(view_vec, vp_Normal);\n"
@@ -1115,9 +1137,9 @@ ShaderGenerator::apply(osg::TexGen* texgen, int unit, GenBuffers& buf)
             break;
 
         case osg::TexGen::REFLECTION_MAP:
-            buf._vertHead
-                << "varying vec3 vp_Normal;\n";
-            buf._vertBody
+            buf._viewHead
+                << "vec3 vp_Normal; // stage global\n";
+            buf._viewBody
                 << INDENT "{\n"
                 << INDENT "vec3 view_vec = normalize(vertex_view.xyz/vertex_view.w);\n"
                 << INDENT TEX_COORD << unit << " = vec4(reflect(view_vec, vp_Normal), 1.0); \n"
@@ -1125,9 +1147,9 @@ ShaderGenerator::apply(osg::TexGen* texgen, int unit, GenBuffers& buf)
             break;
 
         case osg::TexGen::NORMAL_MAP:
-            buf._vertHead
-                << "varying vec3 vp_Normal;\n";
-            buf._vertBody
+            buf._viewHead
+                << "vec3 vp_Normal; //stage global\n";
+            buf._viewBody
                 << INDENT "{\n"
                 << INDENT TEX_COORD << unit << " = vec4(vp_Normal, 1.0); \n"
                 << INDENT "}\n";
@@ -1144,7 +1166,7 @@ ShaderGenerator::apply(osg::TexGen* texgen, int unit, GenBuffers& buf)
         // GLSL only supports built-in "gl_MultiTexCoord{0..7}"
         if ( unit <= 7 )
         {
-            buf._vertBody
+            buf._modelBody
                 << INDENT << TEX_COORD << unit << " = gl_MultiTexCoord" << unit << ";\n";
         }
         else
@@ -1154,7 +1176,7 @@ ShaderGenerator::apply(osg::TexGen* texgen, int unit, GenBuffers& buf)
                 << "requires a custom vertex attribute (osg_MultiTexCoord" << unit << ")."
                 << std::endl;
 
-            buf._vertBody 
+            buf._modelBody 
                 << INDENT << TEX_COORD << unit << " = osg_MultiTexCoord" << unit << ";\n";
         }
     }
@@ -1169,8 +1191,8 @@ ShaderGenerator::apply(osg::TexMat* texmat, int unit, GenBuffers& buf)
     {
         std::string texMatUniform = Stringify() << TEX_MATRIX << unit;
 
-        buf._vertHead << "uniform mat4 " << texMatUniform << ";\n";
-        buf._vertBody << INDENT << TEX_COORD << unit << " = " << texMatUniform << " * " << TEX_COORD<<unit << ";\n";
+        buf._viewHead << "uniform mat4 " << texMatUniform << ";\n";
+        buf._viewBody << INDENT << TEX_COORD << unit << " = " << texMatUniform << " * " << TEX_COORD<<unit << ";\n";
 
         buf._stateSet
             ->getOrCreateUniform(texMatUniform, osg::Uniform::FLOAT_MAT4)
@@ -1184,7 +1206,7 @@ bool
 ShaderGenerator::apply(osg::Texture1D* tex, int unit, GenBuffers& buf)
 {
     buf._fragHead << "uniform sampler1D " SAMPLER << unit << ";\n";
-    buf._fragBody << INDENT "texel = texture1D(" SAMPLER << unit << ", " TEX_COORD << unit << ".x);\n";
+    buf._fragBody << INDENT "texel = texture(" SAMPLER << unit << ", " TEX_COORD << unit << ".x);\n";
     buf._stateSet->getOrCreateUniform( Stringify() << SAMPLER << unit, osg::Uniform::SAMPLER_1D )->set( unit );
 
     return true;
@@ -1194,7 +1216,7 @@ bool
 ShaderGenerator::apply(osg::Texture2D* tex, int unit, GenBuffers& buf)
 {
     buf._fragHead << "uniform sampler2D " SAMPLER << unit << ";\n";
-    buf._fragBody << INDENT "texel = texture2D(" SAMPLER << unit << ", " TEX_COORD << unit << ".xy);\n";
+    buf._fragBody << INDENT "texel = texture(" SAMPLER << unit << ", " TEX_COORD << unit << ".xy);\n";
     buf._stateSet->getOrCreateUniform( Stringify() << SAMPLER << unit, osg::Uniform::SAMPLER_2D )->set( unit );
 
     return true;
@@ -1204,7 +1226,7 @@ bool
 ShaderGenerator::apply(osg::Texture3D* tex, int unit, GenBuffers& buf)
 {
     buf._fragHead << "uniform sampler3D " SAMPLER << unit << ";\n";
-    buf._fragBody << INDENT "texel = texture3D(" SAMPLER << unit << ", " TEX_COORD << unit << ".xyz);\n";
+    buf._fragBody << INDENT "texel = texture(" SAMPLER << unit << ", " TEX_COORD << unit << ".xyz);\n";
     buf._stateSet->getOrCreateUniform( Stringify() << SAMPLER << unit, osg::Uniform::SAMPLER_3D )->set( unit );
 
     return true;
@@ -1213,10 +1235,10 @@ ShaderGenerator::apply(osg::Texture3D* tex, int unit, GenBuffers& buf)
 bool
 ShaderGenerator::apply(osg::TextureRectangle* tex, int unit, GenBuffers& buf)
 {
-    buf._vertHead << "#extension GL_ARB_texture_rectangle : enable\n";
+    buf._viewHead << "#extension GL_ARB_texture_rectangle : enable\n";
 
     buf._fragHead << "uniform sampler2DRect " SAMPLER << unit << ";\n";
-    buf._fragBody << INDENT "texel = texture2DRect(" SAMPLER << unit << ", " TEX_COORD << unit << ".xy);\n";
+    buf._fragBody << INDENT "texel = texture(" SAMPLER << unit << ", " TEX_COORD << unit << ".xy);\n";
     buf._stateSet->getOrCreateUniform( Stringify() << SAMPLER << unit, osg::Uniform::SAMPLER_2D )->set( unit );
 
     return true;
@@ -1228,7 +1250,7 @@ ShaderGenerator::apply(osg::Texture2DArray* tex, int unit, GenBuffers& buf)
     buf._fragHead <<  "#extension GL_EXT_texture_array : enable \n";    
 
     buf._fragHead << "uniform sampler2DArray " SAMPLER << unit << ";\n";
-    buf._fragBody << INDENT "texel = texture2DArray(" SAMPLER << unit << ", " TEX_COORD << unit << ".xyz);\n";
+    buf._fragBody << INDENT "texel = texture(" SAMPLER << unit << ", " TEX_COORD << unit << ".xyz);\n";
     buf._stateSet->getOrCreateUniform( Stringify() << SAMPLER << unit, osg::Uniform::SAMPLER_2D_ARRAY )->set( unit );         
 
     return true;
@@ -1239,7 +1261,7 @@ ShaderGenerator::apply(osg::TextureCubeMap* tex, int unit, GenBuffers& buf)
 {
     std::string sampler = Stringify() << SAMPLER << unit;
     buf._fragHead << "uniform samplerCube " << sampler << ";\n";
-    buf._fragBody << INDENT "texel = textureCube(" << sampler << ", " TEX_COORD << unit << ".xyz);\n";
+    buf._fragBody << INDENT "texel = texture(" << sampler << ", " TEX_COORD << unit << ".xyz);\n";
     buf._stateSet->getOrCreateUniform( sampler, osg::Uniform::SAMPLER_CUBE )->set( unit );         
 
     return true;
@@ -1252,7 +1274,7 @@ ShaderGenerator::apply(osg::PointSprite* tex, int unit, GenBuffers& buf)
 
     std::string sampler = Stringify() << SAMPLER << unit;
     buf._fragHead << "uniform sampler2D " << sampler << ";\n";
-    buf._fragBody << INDENT << "texel = texture2D(" << sampler << ", gl_PointCoord);\n";
+    buf._fragBody << INDENT << "texel = texture(" << sampler << ", gl_PointCoord);\n";
     buf._stateSet->getOrCreateUniform( sampler, osg::Uniform::SAMPLER_2D )->set( unit );
 
     return true;
@@ -1281,3 +1303,19 @@ ShaderGenerator::apply(osg::StateAttribute* attr, GenBuffers& buf)
     // NOP for now.
     return false;
 }
+
+void
+ShaderGenerator::disableUnsupportedAttributes(osg::StateSet* stateset)
+{
+    if (!stateset)
+        return;
+
+#if 0
+#if !defined(OSG_GL_FIXED_FUNCTION_AVAILABLE)
+    stateset->removeAttribute(osg::StateAttribute::TEXENV);
+    stateset->removeAttribute(osg::StateAttribute::TEXENVFILTER);
+    stateset->removeAttribute(osg::StateAttribute::TEXGEN);
+    stateset->removeAttribute(osg::StateAttribute::LIGHTMODEL);
+#endif
+#endif
+}
