@@ -39,6 +39,8 @@ void osgEarth::removeEventHandler(osgViewer::View* view, osgGA::GUIEventHandler*
 
 //------------------------------------------------------------------------
 
+#ifdef OE_HAVE_PIXEL_AUTO_TRANSFORM
+
 #undef LC
 #define LC "[PixelAutoTransform] "
 
@@ -62,6 +64,8 @@ PixelAutoTransform::accept( osg::NodeVisitor& nv )
     if ( !nv.validNodeMask(*this) )
         return;
 
+    bool resetLodScale = false;
+    double oldLodScale = 1.0;
     if ( nv.getVisitorType() == osg::NodeVisitor::CULL_VISITOR )
     {
         // re-activate culling now that the first cull traversal has taken place.
@@ -293,13 +297,27 @@ PixelAutoTransform::accept( osg::NodeVisitor& nv )
             _dirty = false;
 
             // update the LOD Scale based on the auto-scale.
-            cv->setLODScale( 1.0/getScale().x() );
+            const double xScale = getScale().x();
+            if (xScale != 1.0 && xScale != 0.0)
+            {
+                oldLodScale = cv->getLODScale();
+                resetLodScale = true;
+                cv->setLODScale( 1.0/xScale );
+            }
 
         } // if (cv)
     } // if is cull visitor
 
     // finally, skip AT's accept and do Transform.
     Transform::accept(nv);
+
+    // Reset the LOD scale if we changed it
+    if (resetLodScale)
+    {
+        osgUtil::CullVisitor* cv = dynamic_cast<osgUtil::CullVisitor*>(&nv);
+        if ( cv )
+            cv->setLODScale( oldLodScale );
+    }
 }
 
 void
@@ -309,6 +327,8 @@ PixelAutoTransform::dirty()
     setCullingActive( false );
 }
 
+#endif // OE_HAVE_PIXEL_AUTO_TRANSFORM
+
 //-----------------------------------------------------------------------------
 
 #undef  LC
@@ -446,12 +466,11 @@ GeometryValidator::apply(osg::Geometry& geom)
         return;
     }
 
-#if OSG_VERSION_GREATER_OR_EQUAL(3,1,9)
-
     std::set<osg::BufferObject*> _vbos;
 
     osg::Geometry::ArrayList arrays;
     geom.getArrayList(arrays);
+
     for(unsigned i=0; i<arrays.size(); ++i)
     {
         osg::Array* a = arrays[i].get();
@@ -480,34 +499,6 @@ GeometryValidator::apply(osg::Geometry& geom)
         OE_NOTICE << LC << "Found a Geometry that uses more than one VBO (non-optimal sharing)\n";
     }
 
-#else // pre-3.1.9 ... phase out.
-
-    if ( geom.getColorArray() )
-    {
-        if ( geom.getColorBinding() == osg::Geometry::BIND_OVERALL && geom.getColorArray()->getNumElements() != 1 )
-        {
-            OE_NOTICE << "Color: BIND_OVERALL with wrong number of elements" << std::endl;
-        }
-        else if ( geom.getColorBinding() == osg::Geometry::BIND_PER_VERTEX && geom.getColorArray()->getNumElements() != numVerts )
-        {
-            OE_NOTICE << "Color: BIND_PER_VERTEX with colors.size != verts.size" << std::endl;
-        }
-    }
-
-    if ( geom.getNormalArray() )
-    {
-        if ( geom.getNormalBinding() == osg::Geometry::BIND_OVERALL && geom.getNormalArray()->getNumElements() != 1 )
-        {
-            OE_NOTICE << "Normal: BIND_OVERALL with wrong number of elements" << std::endl;
-        }
-        else if ( geom.getNormalBinding() == osg::Geometry::BIND_PER_VERTEX && geom.getNormalArray()->getNumElements() != numVerts )
-        {
-            OE_NOTICE << "Normal: BIND_PER_VERTEX with normals.size != verts.size" << std::endl;
-        }
-    }
-
-#endif
-
     const osg::Geometry::PrimitiveSetList& plist = geom.getPrimitiveSetList();
     
     std::set<osg::BufferObject*> _ebos;
