@@ -22,6 +22,7 @@
 #include <osgEarth/Extension>
 #include <osgEarth/StringUtils>
 #include <osgEarth/FileUtils>
+#include <osgEarth/URI>
 #include <osgDB/FileUtils>
 #include <osgDB/FileNameUtils>
 #include <stdio.h>
@@ -83,17 +84,20 @@ namespace
 		return std::find_first_of(it, end, PATH_SEPARATORS, PATH_SEPARATORS+PATH_SEPARATORS_LEN);
 	}
 
+    // Config tags that we handle specially (versus just letting the plugin mechanism
+    // take take of them)
     bool isReservedWord(const std::string& k)
     {
         return
             k == "options" ||
-            k == "image" ||
+            //k == "image" ||
             k == "elevation" ||
             k == "heightfield" ||
-            k == "model" ||
-            k == "mask" ||
+            //k == "model" ||
+            //k == "mask" ||
             k == "external" ||
-            k == "extensions";
+            k == "extensions" ||
+            k == "libraries";
     }
 
     /**
@@ -314,50 +318,8 @@ namespace
 
 namespace
 {
-    void addImageLayer(const Config& conf, Map* map)
-    {
-        ImageLayerOptions options( conf );
-        options.name() = conf.value("name");
-        ImageLayer* layer = new ImageLayer(options);
-        map->addImageLayer(layer);
-        if (layer->getStatus().isError())
-            OE_WARN << LC << "Layer \"" << layer->getName() << "\" : " << layer->getStatus().toString() << std::endl;
-    }
-
-    void addElevationLayer(const Config& conf, Map* map)
-    {
-        ElevationLayerOptions options( conf );
-        options.name() = conf.value( "name" );
-        ElevationLayer* layer = new ElevationLayer(options);
-        map->addElevationLayer(layer);
-        if (layer->getStatus().isError())
-            OE_WARN << LC << "Layer \"" << layer->getName() << "\" : " << layer->getStatus().toString() << std::endl;
-    }
-
-    void addModelLayer(const Config& conf, Map* map)
-    {
-        ModelLayerOptions options( conf );
-        options.name() = conf.value( "name" );
-        options.driver() = ModelSourceOptions( conf );
-        ModelLayer* layer = new ModelLayer(options);
-        map->addModelLayer(layer);
-        if (layer->getStatus().isError())
-            OE_WARN << LC << "Layer \"" << layer->getName() << "\" : " << layer->getStatus().toString() << std::endl;
-    }
-
-    void addMaskLayer(const Config& conf, Map* map)
-    {
-        MaskLayerOptions options(conf);
-        options.name() = conf.value( "name" );
-        options.driver() = MaskSourceOptions(options);
-        MaskLayer* layer = new MaskLayer(options);
-        map->addTerrainMaskLayer(layer);
-        if (layer->getStatus().isError())
-            OE_WARN << LC << "Layer \"" << layer->getName() << "\" : " << layer->getStatus().toString() << std::endl;
-    }
-
     // support for "special" extension names (convenience and backwards compat)
-    Extension* createSpecialExtension(const Config& conf, MapNode* mapNode)
+    Extension* createSpecialExtension(const Config& conf)
     {
         // special support for the default sky extension:
         if (conf.key() == "sky" && !conf.hasValue("driver"))
@@ -369,7 +331,18 @@ namespace
         return 0L;
     }
 
-    void addExtension(const Config& conf, MapNode* mapNode)
+    bool addLayer(const Config& conf, Map* map)
+    {
+        std::string name = conf.key();
+        Layer* layer = Layer::create(name, conf);
+        if (layer)
+        {
+            map->addLayer(layer);
+        }
+        return layer != 0L;
+    }
+
+    Extension* loadExtension(const Config& conf)
     {
         std::string name = conf.key();
         Extension* extension = Extension::create( conf.key(), conf );
@@ -379,17 +352,27 @@ namespace
             extension = Extension::create(name, conf);
 
             if (!extension)
-                extension = createSpecialExtension(conf, mapNode);
+                extension = createSpecialExtension(conf);
         }
 
-        if (extension)
+        if (!extension)
         {
-            mapNode->addExtension(extension);
-        }
-        else
-        {            
             OE_INFO << LC << "Failed to find an extension for \"" << name << "\"\n";
         }
+
+        return extension;
+    }
+
+    void reportErrors(const Map* map)
+    {
+        for (unsigned i = 0; i < map->getNumLayers(); ++i)
+        {
+            const Layer* layer = map->getLayerAt(i);
+            if (layer->getStatus().isError())
+            {
+                OE_WARN << LC << layer->getTypeName() << " \"" << layer->getName() << "\" : " << layer->getStatus().toString() << std::endl;
+            }
+        }
     }
 }
 
@@ -420,23 +403,32 @@ EarthFileSerializer2::deserialize( const Config& conf, const std::string& referr
         mapOptions.mergeConfig( legacy );
     }
 
-    Map* map = new Map( mapOptions );
+    osg::ref_ptr< Map > map = new Map( mapOptions );
 
-    // Yes, MapOptions and MapNodeOptions share the same Config node. Weird but true.
-    MapNodeOptions mapNodeOptions( conf.child( "options" ) );
-
-    // Create a map node.
-    osg::ref_ptr<MapNode> mapNode = new MapNode( map, mapNodeOptions );
+    // Start a batch update of the map:
+    map->beginUpdate();
 
     // Read all the elevation layers in FIRST so other layers can access them for things like clamping.
+    // TODO: revisit this since we should really be listening for elevation data changes and
+    // re-clamping based on that..
     for(ConfigSet::const_iterator i = conf.children().begin(); i != conf.children().end(); ++i)
     {
-        if ( i->key() == "elevation" || i->key() == "heightfield" )
+        // for backwards compatibility:
+        if (i->key() == "heightfield")
         {
-            addElevationLayer( *i, map );
+            Config temp = *i;
+            temp.key() = "elevation";
+            addLayer(temp, map.get());
+        }
+
+        else if ( i->key() == "elevation" ) // || i->key() == "heightfield" )
+        {
+            addLayer(*i, map.get());
         }
     }
 
+    Config externalConfig;
+    std::vector<osg::ref_ptr<Extension> > extensions;
 
     // Read the layers in LAST (otherwise they will not benefit from the cache/profile configuration)
     for(ConfigSet::const_iterator i = conf.children().begin(); i != conf.children().end(); ++i)
@@ -446,12 +438,13 @@ EarthFileSerializer2::deserialize( const Config& conf, const std::string& referr
             // nop - handled earlier
         }
 
+#if 0
         else if ( i->key() == "image" )
         {
             addImageLayer( *i, map );
         }
 
-        else if ( i->key() == "model" )
+        else */if ( i->key() == "model" )
         {
             addModelLayer( *i, map );
         }
@@ -460,22 +453,60 @@ EarthFileSerializer2::deserialize( const Config& conf, const std::string& referr
         {
             addMaskLayer( *i, map );
         }
+#endif
 
         else if ( i->key() == "external" || i->key() == "extensions" )
         {
-            mapNode->externalConfig() = *i;
+            externalConfig = *i;
+            
             for(ConfigSet::const_iterator e = i->children().begin(); e != i->children().end(); ++e)
             {
-                addExtension( *e, mapNode.get() );
+                Extension* extension = loadExtension(*e);
+                if (extension)
+                    extensions.push_back(extension);
+                //addExtension( *e, mapNode.get() );
             }
         }
 
         else if ( !isReservedWord(i->key()) ) // plugins/extensions.
         {
-            addExtension( *i, mapNode.get() );
+            // try to add as a plugin Layer first:
+            bool addedLayer = addLayer(*i, map.get()); 
+
+            // failing that, try to load as an extension:
+            if ( !addedLayer )
+            {
+                Extension* extension = loadExtension(*i);
+                if (extension)
+                    extensions.push_back(extension);
+            }
         }
     }
 
+    // Complete the batch update of the map
+    map->endUpdate();
+
+    // If any errors occurred, report them now.
+    reportErrors(map.get());
+
+    // Yes, MapOptions and MapNodeOptions share the same Config node. Weird but true.
+    MapNodeOptions mapNodeOptions( conf.child("options") );
+
+    // Create a map node.
+    osg::ref_ptr<MapNode> mapNode = new MapNode( map.get(), mapNodeOptions );
+
+    // Apply the external conf if there is one.
+    if (!externalConfig.empty())
+    {
+        mapNode->externalConfig() = externalConfig;
+    }
+
+    // Install the extensions
+    for (unsigned i = 0; i < extensions.size(); ++i)
+    {
+        mapNode->addExtension(extensions[i].get());
+    }
+
     // return the topmost parent of the mapnode. It's possible that
     // an extension added parents!
     osg::Node* top = mapNode.release();
@@ -491,74 +522,19 @@ Config
 EarthFileSerializer2::serialize(const MapNode* input, const std::string& referrer) const
 {
     Config mapConf("map");
-    mapConf.set("version", "2");
-
-    if ( !input || !input->getMap() )
-        return mapConf; 
-
-    const Map* map = input->getMap();
-    MapFrame mapf( map, Map::ENTIRE_MODEL );
-
-    // the map and node options:
-    Config optionsConf = map->getInitialMapOptions().getConfig();
-    optionsConf.merge( input->getMapNodeOptions().getConfig() );
-    mapConf.add( "options", optionsConf );
-
-    // the layers
-    for( ImageLayerVector::const_iterator i = mapf.imageLayers().begin(); i != mapf.imageLayers().end(); ++i )
+    
+    if (input && input->getMap())
     {
-        ImageLayer* layer = i->get();
-        //Config layerConf = layer->getInitialOptions().getConfig();
-        Config layerConf = layer->getImageLayerOptions().getConfig();
-        layerConf.set("name", layer->getName());
-        layerConf.set("driver", layer->getInitialOptions().driver()->getDriver());        
-        mapConf.add( "image", layerConf );
-    }
+        mapConf = input->getConfig();
 
-    for( ElevationLayerVector::const_iterator i = mapf.elevationLayers().begin(); i != mapf.elevationLayers().end(); ++i )
-    {
-        ElevationLayer* layer = i->get();
-        //Config layerConf = layer->getInitialOptions().getConfig();
-        Config layerConf = layer->getElevationLayerOptions().getConfig();
-        layerConf.set("name", layer->getName());
-        layerConf.set("driver", layer->getInitialOptions().driver()->getDriver());        
-        mapConf.add( "elevation", layerConf );
-    }
-
-    for( ModelLayerVector::const_iterator i = mapf.modelLayers().begin(); i != mapf.modelLayers().end(); ++i )
-    {
-        ModelLayer* layer = i->get();
-        Config layerConf = layer->getModelLayerOptions().getConfig();
-        layerConf.set("name", layer->getName());
-        layerConf.set("driver", layer->getModelLayerOptions().driver()->getDriver());
-        mapConf.add( "model", layerConf );
-    }
-
-    typedef std::vector< osg::ref_ptr<Extension> > Extensions;
-    for(Extensions::const_iterator i = input->getExtensions().begin(); i != input->getExtensions().end(); ++i)
-    {
-        Extension* e = i->get();
-        Config conf = e->getConfigOptions().getConfig();
-        if ( !conf.key().empty() )
+        // Re-write pathnames in the Config so they are relative to the new referrer.
+        if ( _rewritePaths && !referrer.empty() )
         {
-            mapConf.add( conf );
+            RewritePaths rewritePaths( referrer );
+            rewritePaths.setRewriteAbsolutePaths( _rewriteAbsolutePaths );
+            rewritePaths.apply( mapConf );
         }
     }
 
-    Config ext = input->externalConfig();
-    if ( !ext.empty() )
-    {
-        ext.key() = "external";
-        mapConf.add( ext );
-    }
-
-    // Re-write pathnames in the Config so they are relative to the new referrer.
-    if ( _rewritePaths && !referrer.empty() )
-    {
-        RewritePaths rewritePaths( referrer );
-        rewritePaths.setRewriteAbsolutePaths( _rewriteAbsolutePaths );
-        rewritePaths.apply( mapConf );
-    }
-
     return mapConf;
 }
