@@ -1,6 +1,6 @@
 /* -*-c++-*- */
 /* osgEarth - Dynamic map generation toolkit for OpenSceneGraph
- * Copyright 2015 Pelican Mapping
+ * Copyright 2016 Pelican Mapping
  * http://osgearth.org
  *
  * osgEarth is free software; you can redistribute it and/or modify
@@ -627,7 +627,7 @@ class VPBSourceFactory : public TileSourceDriver
             supportsExtension( "osgearth_vpb", "VirtualPlanetBuilder data" );
         }
 
-        virtual const char* className()
+        virtual const char* className() const
         {
             return "VirtualPlanetBuilder ReaderWriter";
         }
