@@ -148,8 +148,8 @@ WCS11Source::createRequest( const TileKey& key ) const
     double lon_min, lat_min, lon_max, lat_max;
     key.getExtent().getBounds( lon_min, lat_min, lon_max, lat_max );
 
-    int lon_samples = _options.tileSize().value();
-    int lat_samples = _options.tileSize().value();
+    int lon_samples = getPixelsPerTile();
+    int lat_samples = getPixelsPerTile();
     double lon_interval = (lon_max-lon_min)/(double)(lon_samples-1);
     double lat_interval = (lat_max-lat_min)/(double)(lat_samples-1);
 
