@@ -151,8 +151,8 @@ public:
             << "&FORMAT=" << ( wmsFormatToUse.empty() ? std::string("image/") + _formatToUse : wmsFormatToUse )
             << "&STYLES=" << _options.style().value()
             << (_options.wmsVersion().value() == "1.3.0" ? "&CRS=" : "&SRS=") << _srsToUse            
-            << "&WIDTH="<< _options.tileSize().value()
-            << "&HEIGHT="<< _options.tileSize().value()
+            << "&WIDTH="<< getPixelsPerTile()
+            << "&HEIGHT=" << getPixelsPerTile()
             << "&BBOX=%lf,%lf,%lf,%lf";
 
         // then the optional keys:
@@ -168,9 +168,9 @@ public:
         osg::ref_ptr<SpatialReference> wms_srs = SpatialReference::create( _srsToUse );
 
         // check for spherical mercator:
-        if ( wms_srs.valid() && wms_srs->isEquivalentTo( osgEarth::Registry::instance()->getGlobalMercatorProfile()->getSRS() ) )
+        if ( wms_srs.valid() && wms_srs->isEquivalentTo( osgEarth::Registry::instance()->getSphericalMercatorProfile()->getSRS() ) )
         {
-            result = osgEarth::Registry::instance()->getGlobalMercatorProfile();
+            result = osgEarth::Registry::instance()->getSphericalMercatorProfile();
         }
         else if (wms_srs.valid() && wms_srs->isEquivalentTo( osgEarth::Registry::instance()->getGlobalGeodeticProfile()->getSRS()))
         {
@@ -246,8 +246,8 @@ public:
                 _formatToUse,
                 _options.style().value(),
                 _srsToUse,
-                _options.tileSize().value(),
-                _options.tileSize().value(),
+                getPixelsPerTile(),
+                getPixelsPerTile(),
                 patterns );
 
             if (patterns.size() > 0)
@@ -448,7 +448,7 @@ public:
             return ImageUtils::createEmptyImage();
         }
 
-        _sequenceCache.insert( seq );
+        _sequenceCache.insert( seq.get() );
         return seq.release();
     }
 
@@ -492,11 +492,6 @@ public:
         return uri;
     }
 
-    virtual int getPixelsPerTile() const
-    {
-        return _options.tileSize().value();
-    }
-
     virtual std::string getExtension()  const 
     {
         return _formatToUse;
