@@ -17,7 +17,7 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>
  */
 #include <osgEarthFeatures/BuildTextFilter>
-//#include <osgEarthFeatures/BuildTextOperator> // this should be in symbology -gw
+#include <osgEarthFeatures/FilterContext>
 #include <osgEarthFeatures/LabelSource>
 #include <osgEarthSymbology/TextSymbol>
 #include <osgText/Text>
