@@ -67,7 +67,7 @@ LabelSource::~LabelSource()
 LabelSource*
 LabelSourceFactory::create( const LabelSourceOptions& options )
 {
-    LabelSource* labelSource = 0L;
+    osg::ref_ptr<LabelSource> source;
 
     if ( !options.getDriver().empty() )
     {
@@ -76,10 +76,11 @@ LabelSourceFactory::create( const LabelSourceOptions& options )
         osg::ref_ptr<osgDB::Options> rwopts = Registry::instance()->cloneOrCreateOptions();
         rwopts->setPluginData( LABEL_SOURCE_OPTIONS_TAG, (void*)&options );
 
-        labelSource = dynamic_cast<LabelSource*>( osgDB::readObjectFile( driverExt, rwopts.get() ) );
-        if ( labelSource )
+        osg::ref_ptr<osg::Object> object = osgDB::readRefObjectFile( driverExt, rwopts.get() );
+        source = dynamic_cast<LabelSource*>( object.release() );
+        if ( source )
         {
-            //modelSource->setName( options.getName() );
+            //source->setName( options.getName() );
             //OE_INFO << "Loaded LabelSource driver \"" << options.getDriver() << "\" OK" << std::endl;
         }
         else
@@ -92,7 +93,7 @@ LabelSourceFactory::create( const LabelSourceOptions& options )
         OE_WARN << LC << "FAIL, illegal null driver specification" << std::endl;
     }
 
-    return labelSource;
+    return source.release();
 }
 
 //------------------------------------------------------------------------
