@@ -93,7 +93,7 @@ void FogEffect::detach()
         osg::ref_ptr<osg::StateSet> stateset;
         if ( (*it).lock(stateset) )
         {
-            detach( stateset );
+            detach( stateset.get() );
             (*it) = 0L;
         }
     }
